package gui
{
	import cocos2d.CCPoint;
	
	import constants.GameConstants;
	import constants.Resources;
	import logic.Card;
	
	import Loom.Animation.Tween;
	import Loom.Animation.EaseType;
	
	/**
	 * The graphics for a card in the deck
	 */
	public class CardGraphics extends Image
	{
		
		/**********************************************************************/
		
		private var m_card:Card 				= null; // the card that this graphics is for
		private var m_shouldStraighten:Boolean	= false;// when moving, should we straighten at the end?
		private var m_shouldReveal:Boolean		= false;// when moving, should we reveal at the end?
		private var m_isStraight:Boolean		= false;// is the card straight, or skewed (fake 3d)?
		private var m_origX:int					= 0.0;	// our original x, when moving and returning
		private var m_origY:int					= 0.0;	// our original y, when moving and returning
		
		/**********************************************************************/
		
		/**
		 * Set the card that this is for
		 */
		public function get card():Card { return this.m_card; }
		public function set card( c:Card ):void
		{
			// clear/set any callbacks
			if( this.m_card != null )
				this.m_card.onReveal -= this._onRevealCard;
			this.m_card = c;
			if( this.m_card != null )
				this.m_card.onReveal += this._onRevealCard;
				
			// kill any tweens
			Tween.killTweensOf( this );
			
			// set our graphics to the card back graphics
			if( this.texture != Resources.SPRITE_CARD_BACK )
				this.texture = Resources.SPRITE_CARD_BACK;
				
			// set our faux 3d look
			this.setSkewX( GameConstants.TABLE_OBJS_SKEW );
			this.scaleY 		= GameConstants.TABLE_OBJS_SCALE;
			this.m_isStraight	= false;
		}
		
		/**
		 * Getter for the skew x property, so we can tween it
		 */
		public function get skewX():Number { return this.getSkewX(); }
		public function set skewX( n:Number ):void
		{
			this.setSkewX( n );
		}
		
		/**********************************************************************/
		
		/**
		 * Creates a new CardGraphics object
		 */
		public function CardGraphics()
		{
			super( Resources.ATLAS_SPRITES, Resources.SPRITE_CARD_BACK );
			
			// set our anchor point to the bottom left
			this.setAnchorPoint( new CCPoint );
			
			// set our initial card to null
			this.card = null;
		}
		
		/**
		 * Destroys the CardGraphics and clears it for garbage collection
		 */
		override public function destroy():void
		{
			super.destroy();
			Tween.killTweensOf( this );
			
			// null our objects
			this.m_card = null;
		}
		
		/**
		 * Animates the card moving a specific coord
		 * @param x The x value to move to
		 * @param y The y value to move to
		 * @param straighten Should we straighten at the end of it?
		 * @param reveal Should we reveal at the end of it?
		 * @param delay The delay before we should move
		 */
		public function moveTo( x:int, y:int, straighten:Boolean, reveal:Boolean, delay:Number = 0.0 ):void
		{
			// reset our values
			this.m_shouldStraighten = straighten;
			this.m_shouldReveal		= reveal;
			
			// remove any tweens
			Tween.killTweensOf( this );
			
			// move there
			var time:Number = 0.5;
			Tween.to( this, time, { "x":x, "ease":EaseType.EASE_OUT, "delay":delay } );
			var t:Tween = Tween.to( this, time, { "y":y, "ease":EaseType.EASE_OUT, "delay":delay } );
			if( this.m_shouldStraighten || this.m_shouldReveal )
				t.onComplete += this._onMoveComplete;
		}
		
		/**
		 * Returns the card to the deck
		 * @param x The deck top x value
		 * @param y The deck top y value
		 */
		public function returnToDeck( x:int, y:int ):void
		{
			// remove any tweens
			Tween.killTweensOf( this );
			
			// if we're not straight, then flatten first
			var time:Number		= 0.5;
			var delay:Number	= ( this.m_isStraight ) ? 0.15 : 0.0;
			if( this.m_isStraight )
				this.flatten( true );
			Tween.to( this, time, { "x":x, "ease":EaseType.EASE_OUT, "delay":delay } );
			Tween.to( this, time, { "y":y, "ease":EaseType.EASE_OUT, "delay":delay } ).onComplete += this._onReturnComplete;
		}
		
		/**
		 * Animates the shuffle for the card
		 */
		public function shuffle():void
		{
			// remove any tweens
			Tween.killTweensOf( this );
			
			// hold our original position
			this.m_origX = this.x;
			this.m_origY = this.y;
			
			// get our new x and y
			var dist:int	= 100;
			var x:int		= this.x - dist + int( Math.random() * dist * 2 );
			var y:int		= this.y - dist + int( Math.random() * dist * 2 );
			
			// tween
			var time:Number = 0.2;
			Tween.to( this, time, { "x":x, "ease":EaseType.EASE_OUT } );
			Tween.to( this, time, { "y":y, "ease":EaseType.EASE_OUT } ).onComplete += this._onShufflePart1Complete;
		}
		
		/**
		 * Straightens the card - if it's for the player/opponent's hole cards
		 * @param animate Should we animate this?
		 */
		public function straighten( animate:Boolean = true ):void
		{
			this.m_isStraight = true;
			if( animate )
			{
				var time:Number = 0.15;
				Tween.killTweensOf( this );
				Tween.to( this, time, { "skewX":0.0, "ease":EaseType.EASE_OUT } );
				Tween.to( this, time, { "scaleY":1.0, "ease":EaseType.EASE_OUT } );
			}
			else
			{
				this.setSkewX( 0.0 );
				this.scaleY = 1.0;
			}
		}
		
		/**
		 * Flattens the card
		 * @param animate Should we animate this?
		 */
		public function flatten( animate:Boolean = true ):void
		{
			this.m_isStraight = false;
			if( animate )
			{
				var time:Number = 0.15;
				Tween.killTweensOf( this );
				Tween.to( this, time, { "skewX":GameConstants.TABLE_OBJS_SKEW, "ease":EaseType.EASE_OUT } );
				Tween.to( this, time, { "scaleY":GameConstants.TABLE_OBJS_SCALE, "ease":EaseType.EASE_OUT } );
			}
			else
			{
				this.setSkewX( GameConstants.TABLE_OBJS_SKEW );
				this.scaleY = GameConstants.TABLE_OBJS_SCALE;
			}
		}
		
		/**
		 * Reveals the card so the main player can see it
		 */
		public function reveal():void
		{
			if( this.m_card == null )
			{
				Console.print( "[CardGraphics] ERROR: Can't reveal the card, as we don't have one" );
				return;
			}
			
			// set our texture
			if( this.texture != this.m_card.graphicsName )
				this.texture = this.m_card.graphicsName;
		}
		
		/**********************************************************************/
		
		// called when our card wants to be revealed
		private function _onRevealCard():void
		{
			this.reveal();
		}
		
		// called when our move tween has completed
		private function _onMoveComplete( t:Tween ):void
		{
			if( this.m_shouldStraighten )
				this.straighten();
			if( this.m_shouldReveal )
				this.reveal();
		}
		
		// called when our return tween has completed
		private function _onReturnComplete( t:Tween ):void
		{
			// reset our graphics and remove us from the stage
			this.card = null;
			if( this.getParent() != null )
				this.getParent().removeChild( this, false );
		}
		
		// called when the first part of the shuffle anim is complete
		private function _onShufflePart1Complete( t:Tween ):void
		{
			var time:Number = 0.2;
			Tween.to( this, time, { "x":this.m_origX, "ease":EaseType.EASE_IN } );
			Tween.to( this, time, { "y":this.m_origY, "ease":EaseType.EASE_IN } ).onComplete += this._onShufflePart2Complete;
		}
		
		// called when the second part of the shuffle anim is complete
		private function _onShufflePart2Complete( t:Tween ):void
		{
			// remove our graphics
			if( this.getParent() != null )
				this.getParent().removeChild( this, false );
		}
	}
}