package gui
{
	import cocos2d.CCPoint;
	
	import constants.Resources;
	import enums.CardDealDest;
	import logic.Card;
	import logic.Deck;
	
	/**
	 * A card stack, that we'll use to shuffle the cards etc
	 */
	public class CardStack extends Image
	{
		
		/**********************************************************************/
		
		private var m_deckTopPos:CCPoint;							// the position for the top of the card stack (struct so can't set null)
		private var m_cardFinalPositions:Vector.<CCPoint> 	= null; // the final positions for the cards
		private var m_cards:Vector.<CardGraphics>			= null;	// the cards that we're playing with
		private var m_cardHighlights:Vector.<CardHighlight>	= null;	// the card highlights
		private var m_index:int								= 0;	// our current index when dealing with individual cards
		
		/**********************************************************************/
		
		/**
		 * The x position for the card stack
		 */
		public override function get x():Number { return super.getPositionX(); }
		public override function set x( n:Number ):void
		{
			super.setPositionX( n );
			this.m_deckTopPos.x = this.x - 2;
		}
		
		/**
		 * The y position for the card stack
		 */
		public override function get y():Number { return super.getPositionY(); }
		public override function set y( n:Number ):void
		{
			super.setPositionY( n );
			this.m_deckTopPos.y = this.y + 26; // 26 is the height of the front of the card stack graphics
		}
		
		/**********************************************************************/
		
		/**
		 * Creates a new CardStack
		 * @param deck The Deck that this stack is for
		 */
		public function CardStack( deck:Deck )
		{
			super( Resources.ATLAS_SPRITES, Resources.SPRITE_CARD_STACK );
			
			// set our anchor point to the bottom left
			this.setAnchorPoint( new CCPoint() );
			
			// create our deck top position
			this.m_deckTopPos = new CCPoint();
			
			// create the positions for the cards (make sure they're in order)
			this.m_cardFinalPositions 								= new Vector.<CCPoint>( CardDealDest.RIVER + 1 );
			this.m_cardFinalPositions[CardDealDest.BURN] 			= new CCPoint( 588, 90 );
			this.m_cardFinalPositions[CardDealDest.OPPONENT_HOLE_1] = new CCPoint( 355, 150 );
			this.m_cardFinalPositions[CardDealDest.OPPONENT_HOLE_2] = new CCPoint( 408, 150 );
			this.m_cardFinalPositions[CardDealDest.PLAYER_HOLE_1] 	= new CCPoint( 285, 10 );
			this.m_cardFinalPositions[CardDealDest.PLAYER_HOLE_2] 	= new CCPoint( 338, 10 );
			this.m_cardFinalPositions[CardDealDest.FLOP_1] 			= new CCPoint( 252, 90 );
			this.m_cardFinalPositions[CardDealDest.FLOP_2] 			= new CCPoint( 300, 90 );
			this.m_cardFinalPositions[CardDealDest.FLOP_3] 			= new CCPoint( 348, 90 );
			this.m_cardFinalPositions[CardDealDest.TURN] 			= new CCPoint( 396, 90 );
			this.m_cardFinalPositions[CardDealDest.RIVER] 			= new CCPoint( 444, 90 );
			
			// we need max 12 cards for the players, burn, and table, but add a few to get a better shuffle anim
			this.m_cards = new Vector.<CardGraphics>();
			for( var i:int = 0; i < deck.numCards / 2; i++ )
				this.m_cards.push( new CardGraphics() );
				
			// create our highlights
			this.m_cardHighlights = new Vector.<CardHighlight>();
			for( i = 0; i < 5; i++ ) // the highlights are used to show the best hand of 5 cards
				this.m_cardHighlights.push( new CardHighlight() );
			
			// add the callbacks to the deck delegates
			deck.onShuffle 			+= this._onShuffle;
			deck.onBurn				+= this._onBurn;
			deck.onDeal				+= this._onDeal;
			deck.onGatherCards		+= this._onGatherCards;
			deck.onShowWinningHand	+= this._onShowWinningHand;
		}
		
		/**
		 * Destroys the CardStack and clears it for garbage collection
		 */
		override public function destroy():void
		{
			super.destroy();
			
			// destroy all our cards
			var len:int = this.m_cards.length;
			for( var i:int = 0; i < len; i++ )
				this.m_cards[i].destroy();
				
			// null our properties
			this.m_cardFinalPositions	= null;
			this.m_cards				= null;
		}
		
		/**
		 * Resets the card stack for a new game
		 */
		public function reset():void
		{
			// remove all our cards
			var len:int = this.m_cards.length;
			for( var i:int = 0; i < len; i++ )
			{
				if( this.m_cards[i].getParent() != null )
					this.m_cards[i].getParent().removeChild( this.m_cards[i], false );
			}
			
			// reset our index
			this.m_index = 0;
		}
		
		/**********************************************************************/
		
		// called when our deck shuffles the cards
		private function _onShuffle( deck:Deck ):void
		{
			this.m_index 	= 0;
			var len:int		= this.m_cards.length;
			for( var i:int = 0; i < len; i++ )
				this._getCard().shuffle();
		}
		
		// called when our deck burns the cards
		private function _onBurn( deck:Deck ):void
		{
			// get a card
			var cardGraphics:CardGraphics 	= this._getCard();
			var pos:CCPoint					= this.m_cardFinalPositions[CardDealDest.BURN];
			cardGraphics.moveTo( pos.x, pos.y, false, false );
		}
		
		// called when our deck deals a card
		private function _onDeal( deck:Deck, card:Card, dest:CardDealDest, delay:Number ):void
		{
			// make sure the destination is good
			if( dest < 0 || dest >= this.m_cardFinalPositions.length )
			{
				Console.print( "Can't deal card as the destination passed (" + dest + ") is no good" );
				return;
			}
			
			// get a card
			var cardGraphics:CardGraphics	= this._getCard();
			cardGraphics.card				= card;
			var pos:CCPoint					= this.m_cardFinalPositions[dest];
			
			// should we straighten at the end?
			var shouldStraighten 	= ( dest >= CardDealDest.OPPONENT_HOLE_1 && dest <= CardDealDest.PLAYER_HOLE_2 );
			var shouldReveal		= ( dest >= CardDealDest.PLAYER_HOLE_1 ); // player's cards, and table cards
			
			// move it
			cardGraphics.moveTo( pos.x, pos.y, shouldStraighten, shouldReveal, delay );
		}
		
		// gathers all the cards back to the deck
		private function _onGatherCards( deck:Deck ):void
		{
			// hide our highlights
			var len:int = this.m_cardHighlights.length;
			for( var i:int = 0; i < len; i++ )
				this.m_cardHighlights[i].hide();
				
			// go through all our cards, and return them to the deck
			len = this.m_cards.length;
			for( i = 0; i < len; i++ )
			{
				if( this.m_cards[i].getParent() != null )
					this.m_cards[i].returnToDeck( this.m_deckTopPos.x, this.m_deckTopPos.y );
			}
		}
		
		// shows the winning hand to the player
		public function _onShowWinningHand( deck:Deck, cards:Vector.<Card> ):void
		{
			// make sure we have exactly 5
			var len:int = this.m_cardHighlights.length;
			if( cards == null || cards.length != len )
			{
				Console.print( "[CardStack] ERROR: Can't highlight winning cards as our destination vector isn't good" );
				return;
			}
			
			// go through and show our highlights
			for( var i:int = 0; i < len; i++ )
			{
				var dest:CardDealDest	= cards[i].dest;
				var pos:CCPoint			= this.m_cardFinalPositions[dest];
				var isStraight:Boolean 	= ( dest >= CardDealDest.OPPONENT_HOLE_1 && dest <= CardDealDest.PLAYER_HOLE_2 );
				this.m_cardHighlights[i].moveTo( pos.x, pos.y, isStraight );
				this.getParent().addChild( this.m_cardHighlights[i] );
			}
		}
		
		// returns a card that we can use
		private function _getCard():CardGraphics
		{
			// get a card
			var card:CardGraphics = this.m_cards[this.m_index];
			
			// position it on top of the deck
			card.x = this.m_deckTopPos.x;
			card.y = this.m_deckTopPos.y;
			if( this.getParent() )
			{
				if( card.getParent() != null )
					card.getParent().removeChild( card, false ); // remove the child so that when we add it, it's always on top
				this.getParent().addChild( card );
			}
			
			// incret our index
			this.m_index++;
			if( this.m_index >= this.m_cards.length )
				this.m_index = 0;
				
			// return our card
			return card;
		}
		
	}
}