package gui
{
	import cocos2d.CCNode;
	import cocos2d.CCPoint;
	import cocos2d.ccColor3B;
	
	import Loom.Animation.Tween;
	
	import constants.GameConstants;
	import constants.Resources;
	
	/**
	 * Define a delegate so we can add callbacks to the money stack
	 */
	public delegate MoneyStackCallback():void;
	
	/**
	 * Creates a new MoneyStack, which represents the amount of cash a player has
	 */
	public class MoneyStack extends CCNode
	{
		
		/**********************************************************************/
		
		/**
		 * Called when we've finished our tweening anim (current amount). It should take
		 * no parameters
		 */
		public var onFinishAnim:MoneyStackCallback = null;
		
		/**
		 * Called when our cash stack is full
		 */
		public var onFull:MoneyStackCallback = null;
		
		/**********************************************************************/
		
		private var m_images:Vector.<Image> = null; // our vector of images
		private var m_txtAmount:TextField	= null;	// the amount of money in the stack
		private var m_pc:int				= 100;	// our percentage height as an int
		private var m_fullStackH:int		= 0;	// the height of a full stack
		private var m_currAmount:int		= 0;	// the current amount that we have
		private var m_currAnimAmount:int	= 0;	// the current amount that we have (animated)
		private var m_maxAmount:int			= 0;	// the max amount that we can have
		private var m_maxHeightPC:int		= 100;	// the maximum % height
		
		/**********************************************************************/
		
		/**
		 * The width of the money stack
		 */
		public function get width():int { return this.m_images[0].width; }
		
		/**
		 * The height of the money stack
		 */
		public function get height():int { return this.m_txtAmount.y; }
		
		/**
		 * The current amount of cash in this money stack
		 */
		public function get currAmount():int { return this.m_currAmount; }
		public function set currAmount( i:int ):void
		{
			this.m_currAmount = ( i < 0 ) ? 0 : ( i > this.m_maxAmount ) ? this.m_maxAmount : i;
			
			// tween us
			Tween.killTweensOf( this );
			Tween.to( this, 0.5, { "currAnimAmount":this.m_currAmount } ).onComplete += this._onTweenFinished;
		}
		
		/**
		 * The current amount of cash in this money stack (animated)
		 */
		public function get currAnimAmount():int { return this.m_currAnimAmount; }
		public function set currAnimAmount( i:int ):void
		{
			i 				= ( i < 0 ) ? 0 : ( i > this.m_maxAmount ) ? this.m_maxAmount : i;
			this.pcHeight	= i / this.m_maxAmount;
		}
		
		/**
		 * The max amount of cash in this memory stack
		 */
		public function get maxAmount():int { return this.m_maxAmount; }
		public function set maxAmount( i:int ):void
		{
			this.m_maxAmount 	= ( i < 0 ) ? 0 : i;
			this.currAmount		= this.currAmount; // reset the current amount
		}
		
		/**
		 * The max pc height we can have. This lets us use 3 money stacks in the game, and have the players'
		 * money stacks be full, but only 50% the possible height of the pot stack. The name probably needs
		 * to be better though
		 */
		public function get maxHeightPC():Number { return this.m_maxHeightPC * 0.01; }
		public function set maxHeightPC( n:Number ):void
		{
			n					= ( n < 0.0 ) ? 0.0 : ( n > 1.0 ) ? 1.0 : n;
			this.m_maxHeightPC 	= int( n * 100 );
			this.pcHeight		= this.pcHeight; // reset the pc height
		}
		
		/**
		 * Sets the percentage height of the money stack
		 */
		public function get pcHeight():Number { return this.m_pc * 0.01 * ( 1.0 / this.maxHeightPC ); }
		public function set pcHeight( n:Number ):void
		{
			// clamp it
			n 			= ( n < 0.0 ) ? 0.0 : ( n > 1.0 ) ? 1.0 : n;
			this.m_pc	= int( n * this.m_maxHeightPC );
			if( this.m_pc == 0 && int( this.m_maxAmount * n ) > 0 )
				this.m_pc = 1; 	// if the amount is > 0, we always, show something, otherwise if our max is 200
								// and our curr is 1, n will be 0.05, and converted to an int, 0
			
			// go through our images
			var len:int = this.m_images.length;
			for( var i:int = 0; i < len; i++ )
			{
				var image:Image 		= this.m_images[i];
				var isVisible:Boolean	= this.m_pc >= ( i * 10 ) + 1;
				image.setVisible( isVisible );
				if( !isVisible )
					continue;
					
				// check should we show a full stack
				if( this.m_pc >= ( i * 10 + 10 ) )
				{
					// if we're not showing a full stack, do so
					if( image.texture != Resources.SPRITE_MONEY_STACK_10 )
					{
						image.texture 	= Resources.SPRITE_MONEY_STACK_10;
						image.y			= this.m_fullStackH + 10 * i; // each stack is 10 notes
					}
					continue;
				}
					
				// get our inner number of notes
				var inner 		= int( this.m_pc % 10 );
				var name:String = "moneyStack" + inner;
				if( image.texture != name )
				{
					image.texture 	= name;
					image.y			= this.m_fullStackH + ( i - 1 ) * 10 + inner;
				}
			}
			
			// set our amount value (animated)
			this.m_currAnimAmount 	= int( this.m_maxAmount * n + 0.5 ); // +0.5 to stop rounding int errors
			this.m_txtAmount.text	= this.m_currAnimAmount + "";
			this.m_txtAmount.x		= this.width * 0.5 - this.m_txtAmount.width * 0.5;
			this.m_txtAmount.y		= this.m_fullStackH + this.m_pc + this.m_txtAmount.height;
			this.m_txtAmount.setVisible( this.m_currAnimAmount != 0 );
		}
		
		/**********************************************************************/
		
		/**
		 * Creates a new MoneyStack object
		 * @param currAmount The current amount that we have
		 * @param maxAmount The max amount that we can have
		 */
		public function MoneyStack( currAmount:int, maxAmount:int )
		{
			// set our anchor point to the bottom left
			this.setAnchorPoint( new CCPoint( 0.0, 0.0 ) );
			
			// create our images
			var len:int		= 10;
			this.m_images 	= new Vector.<Image>();
			for( var i:int = 0; i < len; i++ )
			{
				var image:Image 	= new Image( Resources.ATLAS_SPRITES, Resources.SPRITE_MONEY_STACK_10 );
				this.m_fullStackH	= image.height;
				image.y				= this.m_fullStackH + 10 * i; // each stack is 10 notes
				this.m_images.push( image );
				this.addChild( image );
			}
			
			// create our amount textfield
			this.m_maxAmount		= maxAmount;
			this.m_txtAmount 		= new TextField( "0", Resources.FONT_CURSE, new ccColor3B( 0, 0, 0 ) );
			this.m_txtAmount.scale	= 0.75;
			this.addChild( this.m_txtAmount );
			
			// set our initial current amount (setter)
			this.currAmount = currAmount;
		}
		
		/**
		 * Resets the MoneyStack to 0 (doesn't change the max amount)
		 */
		public function reset():void
		{
			Tween.killTweensOf( this );	// kill any tweens
			this.onFinishAnim	= null;	// clear any delegates
			this.pcHeight		= 0;	// make sure we're not showing
			this.m_currAmount 	= 0; 	// don't use the setter here as we don't want to tween
		}
		
		/**
		 * Destroys the MoneyStack and clears it for garbage collection
		 */
		public function destroy():void
		{
			// remove any tweens
			Tween.killTweensOf( this );
			
			// destroy all our images
			var len:int = this.m_images.length;
			for( var i:int = 0; i < len; i++ )
				this.m_images[i].destroy();
				
			// destroy our amount
			this.m_txtAmount.destroy();
			
			// null our properties
			this.m_images 		= null;
			this.m_txtAmount	= null;
		}
		
		/**********************************************************************/
		
		// called when our current amount tween has finished
		private function _onTweenFinished( t:Tween ):void
		{
			this.onFinishAnim();
			if( this.m_currAmount == this.m_maxAmount )
				this.onFull();
		}
		
	}
}