package gui
{
    import cocos2d.CCScaledLayer;
	
	import Loom.Animation.Tween;
	import Loom.Animation.EaseType;
	
	import constants.Resources;
	
	/**
	 * Create a default callback for the action buttons (check/fold)
	 */
	public delegate ActionCallback():void;
	
	/**
	 * Create a callback for raising
	 */
	public delegate ActionMoneyCallback( amount:int ):void;
	
	/**
	 * The play action buttons controls which buttons the player see during the play screen
	 */
	public class PlayActionButtons
	{
		
		/**********************************************************************/
		
		/**
		 * Callback called when we click on the check button. It should take no parameters
		 */
		public var onCheck:ActionCallback = null;
		
		/**
		 * Callback called when we call the bet that was made. It should take one parameter of type int (amount)
		 */
		public var onCall:ActionMoneyCallback = null;
		
		/**
		 * Callback called when we raise/bet. It should take one parameter of type int (amount)
		 */
		public var onRaise:ActionMoneyCallback = null;
		
		/**
		 * Callback called when we fold. It should take no parameters
		 */
		public var onFold:ActionCallback = null;
		
		/**********************************************************************/
		
		private var m_guiCanvas:CCScaledLayer	= null; // the gui canvas to add all the buttons to
		private var m_btnCheck:TextBtn 			= null;	// the check button
		private var m_btnRaise:TextBtn			= null;	// the bet/raise button
		private var m_btnOK:Btn					= null;	// the ok button, to say ok to the amount in the raise button
		private var m_btnFold:TextBtn			= null;	// the fold button
		private var m_currRaise:int				= 0;	// the current raise;
		private var m_bigBlind:int				= 0;	// the big blind, which we use to calculate our raises
		private var m_opponentBet:int			= 0;	// the bet that the opponent made, if any
		private var m_maxBet:int				= 0;	// the max bet that we can make
		private var m_alreadyPaid:int			= 0;	// how much we've already paid (e.g. blinds)
		
		/**********************************************************************/
		
		/**
		 * The canvas to add all our buttons to
		 */
		public function set guiCanvas( c:CCScaledLayer ):void
		{
			this.m_guiCanvas = c;
			this._positionButtons();
		}
		
		/**
		 * The big blind, which we use to calculate our raises
		 */
		public function set bigBlind( i:int ):void
		{
			this.m_bigBlind = ( i < 0 ) ? 0 : i;
		}
		
		/**
		 * The max bet that we can make
		 */
		public function set maxBet( i:int ):void
		{
			this.m_maxBet = ( i < 0 ) ? 0 : i;
		}
		
		/**********************************************************************/
		
		/**
		 * Creates the buttons that we use to act during the play screen
		 */
		public function PlayActionButtons()
		{
			// create our buttons
			this.m_btnCheck = new TextBtn( "Check", Resources.SPRITE_BTN_CHECK, Resources.SPRITE_BTN_CHECK_DOWN, Resources.ATLAS_BUTTONS );
			this.m_btnRaise = new TextBtn( "Raise", Resources.SPRITE_BTN_RAISE, Resources.SPRITE_BTN_RAISE_DOWN, Resources.ATLAS_BUTTONS );
			this.m_btnOK	= new Btn( Resources.SPRITE_BTN_OK, Resources.SPRITE_BTN_OK_DOWN, Resources.ATLAS_BUTTONS );
			this.m_btnFold	= new TextBtn( "Fold :(", Resources.SPRITE_BTN_FOLD, Resources.SPRITE_BTN_FOLD_DOWN, Resources.ATLAS_BUTTONS );
			
			// add our callbacks
			this.m_btnCheck.onBtnClick	+= this._onClickCheck;
			this.m_btnRaise.onBtnClick	+= this._onClickRaise;
			this.m_btnOK.onBtnClick		+= this._onClickOK;
			this.m_btnFold.onBtnClick	+= this._onClickFold;
		}
		
		/**
		 * Destroys the PlayActionButtons and clears it for garbage collection
		 */
		public function destroy():void
		{
			// destroy our buttons
			this.m_btnCheck.destroy();
			this.m_btnRaise.destroy();
			this.m_btnOK.destroy();
			this.m_btnFold.destroy();
			
			// null our properties
			this.m_btnCheck	= null;
			this.m_btnRaise	= null;
			this.m_btnOK	= null;	
			this.m_btnFold	= null;
		}
		
		/**
		 * Shows the buttons
		 * @param opponentBet The amount that the oppnonent bet, if any
		 * @param alreadyPaid If we've already paid some of it (e.g. blinds)
		 * @param isOpponentAllIn Is the opponent all in?
		 */
		public function show( opponentBet:int = 0, alreadyPaid:int = 0, isOpponentAllIn:Boolean = false ):void
		{
			// make sure we have a canvas
			if( this.m_guiCanvas == null )
			{
				Console.print( "[PlayActionButtons] ERROR: Can't show the buttons as we don't have our gui canvas set" );
				return;
			}
			
			// set how much we've already paid
			this.m_alreadyPaid = alreadyPaid;
			
			// set our text/current raise etc
			this.m_opponentBet = opponentBet;
			if( this.m_opponentBet <= 0 || opponentBet == this.m_alreadyPaid )
			{
				// here, the opponent either checked, we're going first, or in the case of (opponentBet == this.m_alreadyPaid)
				// it's the pre-flop, the player is the big blind, and the opponent has called - we get another chance to
				// check/raise it
				
				// get our current raise
				this.m_currRaise		= this.m_bigBlind;
				if( this.m_opponentBet != 0 && this.m_opponentBet == this.m_alreadyPaid )
					this.m_currRaise = this.m_bigBlind * 2;
					
				// set our buttons
				this.m_btnCheck.text 	= "Check";
				if( this.m_alreadyPaid == 0 )
					this.m_btnRaise.text 	= "Raise: " + this.m_currRaise;
				else
				{
					var plus:String			= " (+" + ( this.m_currRaise - this.m_alreadyPaid ) + ")";
					this.m_btnRaise.text	= "Raise: " + this.m_currRaise + plus;
				}
				this.m_btnRaise.enabled	= !isOpponentAllIn; // if the opponent's all-in, then raising it does nothing
				this.m_btnOK.enabled	= !isOpponentAllIn;
			}
			else
			{
				// here, the opponent bet is bigger than ours, so they've bet/raised us. we can call
				// or raise again, but it's double their bet
				
				// if the opponent bet is bigger than our max, then we can't raise, just call
				var isAllIn:Boolean		= ( this.m_opponentBet >= this.m_maxBet );
				this.m_currRaise		= this.m_opponentBet * 2;
				if( this.m_currRaise > this.m_maxBet )
					this.m_currRaise 	= this.m_maxBet;
				this.m_btnRaise.enabled	= !isAllIn && !isOpponentAllIn; // also check if the opp is all-in, in which case we can't raise
				this.m_btnOK.enabled	= !isAllIn && !isOpponentAllIn;
				
				// set our button text
				if( isAllIn )
				{
					this.m_btnCheck.text 	= "Call: " + this.m_maxBet + " (all in)";
					this.m_btnRaise.text	= "Raise: all in";
				}
				else
				{
					// if we've already paid something, show the difference
					if( this.m_alreadyPaid > 0 )
					{
						var plus1:String		= " (+" + ( this.m_opponentBet - this.m_alreadyPaid ) + ")";
						var plus2:String		= " (+" + ( this.m_currRaise - this.m_alreadyPaid ) + ")";
						this.m_btnCheck.text	= "Call: " + this.m_opponentBet + plus1;
						this.m_btnRaise.text	= "Raise: " + this.m_currRaise + plus2;
					}
					else
					{
						this.m_btnCheck.text	= "Call: " + this.m_opponentBet;
						this.m_btnRaise.text	= "Raise: " + this.m_currRaise;
					}
				}
			}
			
			// hide our buttons off to the side
			this.m_btnCheck.x 	= -this.m_btnCheck.width;
			this.m_btnRaise.x 	= -this.m_btnRaise.width;
			this.m_btnOK.x 		= -this.m_btnOK.width;
			this.m_btnFold.x 	= -this.m_btnFold.width;
			
			// add our buttons
			if( this.m_btnCheck.getParent() != this.m_guiCanvas )
				this.m_guiCanvas.addChild( this.m_btnCheck );
			if( this.m_btnOK.getParent() != this.m_guiCanvas )
				this.m_guiCanvas.addChild( this.m_btnOK ); // put the ok button behind the raise, as they're on the same y level
			if( this.m_btnRaise.getParent() != this.m_guiCanvas )
				this.m_guiCanvas.addChild( this.m_btnRaise );
			if( this.m_btnFold.getParent() != this.m_guiCanvas )
				this.m_guiCanvas.addChild( this.m_btnFold );
			
			// kill any previous tweens
			Tween.killTweensOf( this.m_btnCheck );
			Tween.killTweensOf( this.m_btnRaise );
			Tween.killTweensOf( this.m_btnOK );
			Tween.killTweensOf( this.m_btnFold );
			
			// tween them in
			var time:Number = 0.75;
			var padding:int	= 5;
			Tween.to( this.m_btnCheck, time, { "x":10, "ease":EaseType.EASE_OUT_BACK } );
			Tween.to( this.m_btnRaise, time, { "x":10, "ease":EaseType.EASE_OUT_BACK, "delay":0.1 } );
			Tween.to( this.m_btnOK, time, { "x":10 + this.m_btnRaise.width + padding, "ease":EaseType.EASE_OUT_BACK, "delay":0.2 } );
			Tween.to( this.m_btnFold, time, { "x":10, "ease":EaseType.EASE_OUT_BACK, "delay":0.3 } );
		}
		
		/**
		 * Hides the buttons
		 */
		public function hide():void
		{			
			// kill any previous tweens
			Tween.killTweensOf( this.m_btnCheck );
			Tween.killTweensOf( this.m_btnRaise );
			Tween.killTweensOf( this.m_btnOK );
			Tween.killTweensOf( this.m_btnFold );
			
			// tween out
			var time:Number = 0.5;
			Tween.to( this.m_btnCheck, time, { "x":-this.m_btnCheck.width, "ease":EaseType.EASE_IN_BACK } );
			Tween.to( this.m_btnRaise, time, { "x":-this.m_btnRaise.width, "ease":EaseType.EASE_IN_BACK, "delay":0.1 } );
			Tween.to( this.m_btnOK, time, { "x":-this.m_btnOK.width, "ease":EaseType.EASE_IN_BACK, "delay":0.2 } );
			Tween.to( this.m_btnFold, time, { "x":-this.m_btnFold.width, "ease":EaseType.EASE_IN_BACK, "delay":0.3 } ).onComplete += this._onTweenOutComplete;
		}
		
		/**********************************************************************/
		
		// positions all our buttons
		private function _positionButtons():void
		{
			// make sure we have a gui canvas
			if( this.m_guiCanvas == null )
			{
				Console.print( "[PlayActionButtons] ERROR: Can't position our buttons as we don't have a gui canvas" );
				return;
			}
			
			// position our buttons
			var padding:int		= 5;
			this.m_btnCheck.x 	= 10;
			this.m_btnCheck.y 	= this.m_guiCanvas.designHeight - 10;;
			this.m_btnRaise.x	= this.m_btnCheck.x;
			this.m_btnRaise.y	= this.m_btnCheck.y - this.m_btnCheck.height - padding;
			this.m_btnOK.x		= this.m_btnRaise.x + this.m_btnRaise.width + padding;
			this.m_btnOK.y		= this.m_btnRaise.y - this.m_btnRaise.height * 0.5 + this.m_btnOK.height * 0.5; // center it with btnRaise
			this.m_btnFold.x	= this.m_btnRaise.x;
			this.m_btnFold.y	= this.m_btnRaise.y - this.m_btnRaise.height - padding;
		}
		
		// called when our tween out is completed - hide our buttons
		private function _onTweenOutComplete( t:Tween ):void
		{			
			// remove all our buttons
			if( this.m_btnCheck.getParent() != null )
				this.m_btnCheck.getParent().removeChild( this.m_btnCheck, false );
			if( this.m_btnRaise.getParent() != null )
				this.m_btnRaise.getParent().removeChild( this.m_btnRaise, false );
			if( this.m_btnOK.getParent() != null )
				this.m_btnOK.getParent().removeChild( this.m_btnOK, false );
			if( this.m_btnFold.getParent() != null )
				this.m_btnFold.getParent().removeChild( this.m_btnFold, false );
		}
		
		// called when we click on the check button
		private function _onClickCheck( b:Btn ):void
		{
			// TODO: if the opponent has bet and the player is calling, call onCall() instead
			if( this.m_opponentBet > 0 )
			{
				var bet:int = ( this.m_opponentBet > this.m_maxBet ) ? this.m_maxBet : this.m_opponentBet;
				this.onCall( bet );
			}
			else
				this.onCheck();
		}
		
		// called when we click on the raise button
		private function _onClickRaise( b:Btn ):void
		{
			// check against our max
			if( this.m_currRaise == this.m_maxBet )
				this.m_currRaise = ( this.m_opponentBet == 0 ) ? this.m_bigBlind : this.m_opponentBet * 2; // always double the opponent's
			else
			{
				var oppBet:int		= this.m_opponentBet; // doing a ternary followed by "this." fucks up
				this.m_currRaise 	+= ( this.m_opponentBet == 0 ) ? this.m_bigBlind : oppBet;
			}
				
			// cap it
			if( this.m_currRaise > this.m_maxBet )
				this.m_currRaise = this.m_maxBet;
			var plus:String 		= ( this.m_alreadyPaid > 0 ) ? " (+" + ( this.m_currRaise - this.m_alreadyPaid ) + ")" : "";
			this.m_btnRaise.text 	= "Raise: " + ( ( this.m_currRaise < this.m_maxBet ) ? this.m_currRaise + plus : "all in" );
		}
		
		// called when we click on the ok button
		private function _onClickOK( b:Btn ):void
		{	
			this.onRaise( this.m_currRaise );
		}
		
		// called when we click on the fold button
		private function _onClickFold( b:Btn ):void
		{
			this.onFold();
		}
		
	}
}