package gui
{
	import cocos2d.CCPoint;
	
	import constants.Resources;
	import logic.AIPlayer;
	import logic.animations.AnimController;
	
	/** 
	 * The graphics for a player in a match
	 */
	public class PlayerGraphics extends Image
	{
		
		/**********************************************************************/
		
		private var m_aiPlayer:AIPlayer 	= null; // the player that this is for
		private var m_speech:SpeechBubble	= null;	// the speech bubble so we can see what the player is thinking
		private var m_anims:AnimController 	= null;	// the animation controller for these graphics
		private var m_entered:Boolean		= false;// have we entered yet?
		
		/**********************************************************************/
		
		/**
		 * The Player that these graphics are for
		 */
		public function get aiPlayer():AIPlayer { return this.m_aiPlayer; }
		public function set aiPlayer( p:AIPlayer ):void
		{
			if( this.m_aiPlayer == p )
				return;
				
			// if we already have an ai player, remove our callbacks
			if( this.m_aiPlayer != null )
			{
				this.m_aiPlayer.onTalk 			-= this._onPlayerTalk;
				this.m_aiPlayer.onStopTalking	-= this._onPlayerStopTalking;
				this.m_aiPlayer.onPlayAnim		-= this._onPlayerPlayAnim;
			}
			this.m_aiPlayer = p;
			if( this.m_aiPlayer == null )
				return;
				
			// add our callbacks
			this.m_aiPlayer.onTalk 			+= this._onPlayerTalk;
			this.m_aiPlayer.onStopTalking	+= this._onPlayerStopTalking;
			this.m_aiPlayer.onPlayAnim		+= this._onPlayerPlayAnim;
			
			// change our texture
			this.texture = this.m_aiPlayer.playerGraphics;
			
			// tell our anim controller
			this.m_anims.aiPlayer = this.m_aiPlayer;
		}
		
		/**********************************************************************/
		
		/**
		 * Creates a new PlayerGraphics object
		 */
		public function PlayerGraphics()
		{
			super( Resources.ATLAS_SPRITES, Resources.SPRITE_DEFAULT_PLAYER );
			
			// set the anchor point to the bottom left
			this.setAnchorPoint( new CCPoint );
			
			// create and add our speech bubble
			this.m_speech 	= new SpeechBubble();
			this.m_speech.x	= -this.m_speech.width;
			this.m_speech.y	= this.height;
			this.addChild( this.m_speech );
			
			// create our animations controller
			this.m_anims = new AnimController( this );
		}
		
		/**
		 * Destroys the PlayerGraphics and clears it for garbage collection
		 */
		override public function destroy():void
		{
			super.destroy();
			
			// destroy our objects
			this.m_speech.destroy();
			this.m_anims.destroy();
			
			// null our properties
			this.aiPlayer 	= null; // call the setter so we clear our callbacks
			this.m_speech	= null;
			this.m_anims	= null;
		}
		
		/**
		 * Called when our graphics have entered the match
		 */
		public function onEnter():void
		{
			// tell our anim controller to record the original position
			this.m_anims.setOriginalPos( this.x, this.y );
			this.m_entered = true;
		}
		
		/**
		 * Called just before we start leaving - stop any anims
		 */
		public function onLeaving():void
		{
			this.m_entered = false;
			this.m_anims.resetPlayer();
		}
		
		/**********************************************************************/
		
		// called when the player wants to say something
		private function _onPlayerTalk( msg:String, autoHide:Boolean ):void
		{
			this.m_speech.show( msg );
			if( autoHide )
				this.m_speech.hide();
		}
		
		// called when the player wants to stop talking
		private function _onPlayerStopTalking():void
		{
			this.m_speech.hide();
		}
		
		// called when the player wants to play an anim
		private function _onPlayerPlayAnim( anim:String ):void
		{
			// do nothing if we haven't entered yet, as we don't have the final position for our
			// graphics, so the animations will bug
			if( !this.m_entered )
				return;
			this.m_anims.playAnim( anim );
		}
	}
}