package gui.popups
{
	import cocos2d.ccColor3B;

	import Loom.Animation.Tween;
	import Loom.Animation.EaseType;
	
	import constants.Resources;
	import gui.Btn;
	import gui.Image;
	import gui.TextBtn;
	import gui.TextField;
	
	/**
	 * Define a delegate for a popup callback
	 */
	public delegate PopupCallback():void;
	
	/**
	 * The popup shown at the end of the match, when we win or lose against an opponent
	 */
	public class Popup extends Image
	{
		
		/**********************************************************************/
		
		/**
		 * Called when we're being hidden. It should take no parameters
		 */
		public var onHide:PopupCallback = null;
		
		/**
		 * Should this popup auto-remove itself from the canvas when hidden? Default is true
		 */
		public var autoRemove:Boolean = true;
		
		/**********************************************************************/
		
		protected var m_txtTitle:TextField	= null;	// the main title textfield
		protected var m_btnOK:TextBtn		= null;	// the ok button
		protected var m_canvasWidth:Number	= 0.0;	// the width of the canvas
		protected var m_isShowing:Boolean	= false;// are we showing?
		
		/**********************************************************************/
		
		/**
		 * Creates a new Popup
		 * @param atlasID The ID of the atlas texture for this popup
		 * @param texture Our texture ID
		 * @param canvasWidth The width of the canvas
		 * @param canvasHeight The height of the canavs
		 */
		public function Popup( atlasID:String, texture:String, canvasWidth:Number, canvasHeight:Number )
		{
			super( atlasID, texture );
			this.m_canvasWidth = canvasWidth;
			
			var padding:Number = 20.0;
			
			// create our title textfield
			this.m_txtTitle 		= new TextField( "Title", Resources.FONT_CURSE, new ccColor3B( 0, 0, 0 ) );
			this.m_txtTitle.scale	= 0.5;
			this.m_txtTitle.x		= this.width * 0.5 - this.m_txtTitle.width * 0.5;
			this.m_txtTitle.y		= this.height - padding;
			
			// create our ok button
			this.m_btnOK 			= new TextBtn( "OK", Resources.SPRITE_BTN_CHECK, Resources.SPRITE_BTN_CHECK_DOWN, Resources.ATLAS_BUTTONS );
			this.m_btnOK.onBtnClick	+= this._onClickOK;
			this.m_btnOK.x			= this.width * 0.5 - this.m_btnOK.width * 0.5;
			this.m_btnOK.y			= this.m_btnOK.height + padding;
			
			// add them all
			this.addChild( this.m_txtTitle );
			this.addChild( this.m_btnOK );
			
			// hide us to begin with
			this.x = -this.width;
			this.y = canvasHeight * 0.5 + this.height * 0.5;
		}
		
		/**
		 * Destroys the Popup and clears it for garbage collection
		 */
		override public function destroy():void
		{
			super.destroy();
			
			// kill any tweens
			Tween.killTweensOf( this );
			
			// destroy our objects
			this.m_txtTitle.destroy();
			this.m_btnOK.destroy();
			
			// null our properties
			this.m_txtTitle	= null;
			this.m_btnOK	= null;
		}
		
		/**
		 * Shows the Popup
		 */
		public function show():void
		{
			this.m_isShowing = true;
			
			// position and tween us in
			this.x = -this.width;
			Tween.killTweensOf( this );
			Tween.to( this, 0.5, { "x":this.m_canvasWidth * 0.5 - this.width * 0.5, "ease":EaseType.EASE_OUT_BACK } );
		}
		
		/**
		 * Hides the Popup
		 */
		public function hide():void
		{
			if( !this.m_isShowing )
				return;
			this.m_isShowing = false;
			
			// tween us out
			Tween.killTweensOf( this );
			var t:Tween = Tween.to( this, 0.5, { "x":this.m_canvasWidth, "ease":EaseType.EASE_IN_BACK } );
			if( this.autoRemove )
				t.onComplete += this._onHidden;
			
			// call our delegate
			this.onHide();
		}
		
		/**********************************************************************/
		
		// called when we click on the ok button
		private function _onClickOK( b:Btn ):void
		{
			this.hide();
		}
		
		// called when we're hidden
		private function _onHidden( t:Tween ):void
		{
			if( this.getParent() != null )
				this.getParent().removeChild( this, false );
		}
		
	}
}