package logic
{
	import Loom.GameFramework.ITicked;
	import Loom.GameFramework.TimeManager;
	
	import constants.Animations;
	import enums.BetRound;
	import enums.CardValue;
	import enums.HandRank;
	import enums.ResultType;
	
	/**
	 * Create a default callback for the ai (check/fold)
	 */
	public delegate AICallback():void;
	
	/**
	 * Create a callback for raising
	 */
	public delegate AIMoneyCallback( amount:int ):void;
	
	/**
	 * The AI class, that controls what the ai player will do
	 */
	public class AI implements ITicked
	{
		
		/**********************************************************************/
		
		/**
		 * Callback called when we decide to check. It should take no parameters
		 */
		public var onCheck:AICallback = null;
		
		/**
		 * Callback called when we call a bet that was made. It should take one parameter of type int (amount)
		 */
		public var onCall:AIMoneyCallback = null;
		
		/**
		 * Callback called when we raise/bet. It should take one parameter of type int (amount)
		 */
		public var onRaise:AIMoneyCallback = null;
		
		/**
		 * Callback called when we fold. It should take no parameters
		 */
		public var onFold:AICallback = null;
		
		/**********************************************************************/
		
		private var m_timeManager:TimeManager	= null;	// the time manager to add us to
		private var m_mainPlayer:MainPlayer		= null;	// the main player
		private var m_aiPlayer:AIPlayer			= null;	// the ai player that this is for
		private var m_active:Boolean 			= false;// are we active?
		private var m_shouldPlayAnim:Boolean	= false;// have we played an anim?
		private var m_currTime:Number			= 0.0;	// the current wait time (so our decisions aren't instantaneous)
		private var m_waitTime:Number			= 0.0;	// the total wait time
		private var m_myBet:int					= 0;	// how much have I bet so far?
		private var m_opponentBet:int			= 0;	// how much has my opponent bet?
		private var m_betRound:BetRound			= BetRound.NONE;	// the current betting round that we're in
		private var m_rangeAggressive:Number	= 0.0;	// our aggressive range, for when we're deciding what to do
		private var m_rangeNormal:Number		= 0.0;	// our normal range, for when we're deciding what to do
		private var m_rangeCautious:Number		= 0.0;	// our cautious range, for when we're deciding what to do
		
		/**********************************************************************/
		
		/**
		 * Sets the AIPlayer that this is for
		 */
		public function set aiPlayer( a:AIPlayer ):void
		{
			this.m_aiPlayer = a;
			if( this.m_aiPlayer == null )
				return;
			
			// set our ranges. our AI basically works like this: the ai player has an aggressiveness
			// property, which we'll set to our m_rangeAggressive. we'll then split the rest between
			// the m_rangeNormal and m_rangeCautious. when we're deciding what to do, we go down
			// the aggressive-normal-cautious route, adjusted by the intelligence of our player, and based
			// on the current betting round (i.e. if it's the river, and our player is intelligent, and
			// losing, they're more likely to fold than call/raise). depending on which route we decide
			// to take, and if we're going first or second, we then decide what we're going to do
			this.m_rangeAggressive 	= this.m_aiPlayer.aggressiveness;
			this.m_rangeCautious	= 1.0;
			this.m_rangeNormal		= this.m_rangeAggressive + ( ( this.m_rangeCautious - this.m_rangeAggressive ) * 0.5 );
		}
		
		/**
		 * Sets the main player so we can check against it
		 */
		public function set mainPlayer( m:MainPlayer ):void
		{
			this.m_mainPlayer = m;
		}
		
		/**********************************************************************/
		
		/**
		 * Creates a new AI class
		 * @param timeManager The main TimeManager class, so we can update
		 */
		public function AI( timeManager:TimeManager ):void
		{
			this.m_timeManager = timeManager;
			this.m_timeManager.addTickedObject( this );
		}
		
		/**
		 * Destroys the AI and clears it for garbage collection
		 */
		public function destroy():void
		{
			// remove us from the time manager
			this.m_timeManager.removeTickedObject( this );
			
			// clear our properties
			this.onCheck		= null;
			this.onCall			= null;
			this.onRaise		= null;
			this.onFold			= null;
			this.m_timeManager	= null;
			this.m_aiPlayer		= null;
			this.m_mainPlayer	= null;
		}
		
		/**
		 * Called every tick
		 */
		public function onTick():void
		{
			if( !this.m_active )
				return;
				
			// wait for a bit
			var dt:Number	= this.m_timeManager.TICK_RATE;
			this.m_currTime	+= dt;
			
			// if we should play an anim
			if( this.m_shouldPlayAnim && this.m_currTime > 0.2 )
			{
				this.m_shouldPlayAnim = false;
				
				// we're either playing the check cards or check pot anim. the check cards anim takes
				// around 3.5 seconds, while the check pot anim takes around 1.5, so pick one
				var anim:String = Animations.CHECK_POT;
				if( this.m_waitTime >= 2.5 )
					anim = Animations.CHECK_CARDS;
					
				// don't play the check pot anim during pre-flop or flop
				if( anim == Animations.CHECK_POT && this.m_betRound <= BetRound.FLOP )
					anim = null;
					
				// there's only a chance that we'll play it
				if( anim != null && Math.random() < 0.4 )
					this.m_aiPlayer.playAnim( anim );
			}
			
			// if we're not finished yet, do nothing
			if( this.m_currTime < this.m_waitTime )
				return;
				
			// we're not active anymore
			this.m_active = false;
			
			// check if we're beating the main player
			var resultType:ResultType = this.m_aiPlayer.cardMatchResults.compareAgainst( this.m_mainPlayer.cardMatchResults );
			
			// get a rand value to decide what to do
			var rand:Number = Math.random();
			
			// hold our ranges
			var ra:Number = this.m_rangeAggressive;
			var rn:Number = this.m_rangeNormal;
			var rc:Number = this.m_rangeCautious;
			
			// modify our ranges based on our intelligence, if we're winning/losing, and the current round.
			
			// if this is the river, and we don't have anything, then check/fold - unless one of our
			// cards is an ace or a king, where we can pretend to be playing the high card
			if( this.m_betRound == BetRound.RIVER && 
				this.m_aiPlayer.cardMatchResults.type == HandRank.HIGH_CARD &&
				this.m_aiPlayer.holeCard1.value < CardValue.KING &&
				this.m_aiPlayer.holeCard2.value < CardValue.KING )
			{
				// we'll default to the cautious ratio
				ra = 0.0;
				rn = 0.0;
			}
			
			// modify the percentage based on the intelligence of the player, the current round, and if we're winning or not.
			// this means that if a player has full intelligence, they're more likely to be aggressive on the river if they're
			// winning, and cautious if they're not (fake reading the player). This still gives us room to bluff
			var modif:Number = 0.05 * this.m_aiPlayer.intelligence * ( this.m_betRound - 1 ) * ( ( resultType == ResultType.LOSE ) ? -1.0 : 1.0 );
			ra				+= modif;
			rn				+= modif;
				
			// if my bet < opponent bet, then the player has bet something and we either call, raise or fold. else
			// the player has bet nothing and we either check or bet
			if( this.m_myBet < this.m_opponentBet )
			{
				// the player has bet something, so we're either going to fold, call, or raise.
				// NOTE: if raising, check the player's money levels - there's no point raising
				// if they're all-in
				if( rand <= ra && this.m_mainPlayer.money > 0 )
					this._raise( this.m_opponentBet * 2 );	// aggressive - raise opponent's bet
				else if( rand <= rn )
					this._call( this.m_opponentBet );		// normal - call
				else
					this._fold();							// cautious - fold
			}
			else
			{
				// the player has bet nothing, so we're either going to check, bet, or raise, depending on our aggressivness.
				// note: we can also be in this, if we're the dealer and the player called us - we get another chance
				var raiseAggressive:int = ( this.m_betRound == BetRound.PRE_FLOP ) ? this.m_aiPlayer.bigBlind * 3 : this.m_aiPlayer.bigBlind * 2;
				var raiseNormal:int		= ( this.m_betRound == BetRound.PRE_FLOP ) ? this.m_aiPlayer.bigBlind * 2 : this.m_aiPlayer.bigBlind;
				
				if( rand <= ra )
					this._raise( raiseAggressive ); 	// aggressive - raise bigBlind * 2
				else if( rand <= rn )
					this._raise( raiseNormal ); 		// normal - raise bigBlind
				else
					this._check();						// cautious - check
			}
		}
		
		/**
		 * Starts thinking
		 * @param myBet How much I've bet so far
		 * @param opponentBet How much my opponent has bet
		 * @param round The current round that we're in
		 */
		public function think( myBet:int, opponentBet:int, round:BetRound ):void
		{
			// make sure we have an ai player
			if( this.m_aiPlayer == null )
			{
				Console.print( "[AI] ERROR: Can't activate as we don't have an AIPlayer set" );
				return;
			}
			this.m_aiPlayer.say( "...", false );
			
			// set our bets
			this.m_myBet		= myBet;
			this.m_opponentBet	= opponentBet;
			this.m_betRound		= round;
			
			// reset our current time, choose a new wait time, and activate
			this.m_currTime 		= 0.0;
			this.m_waitTime			= 1.0 + Math.random() * this.m_betRound;
			this.m_active			= true;
			this.m_shouldPlayAnim	= true;
		}
		
		/**
		 * Should we reveal our hole cards?
		 * @return True if we should reveal them, false otherwise
		 */
		public function shouldRevealHoleCards():Boolean
		{
			// NOTE: this only comes into play if the player folded - we can show our
			// cards to show that we had a good hand, thus it was a good fold, or that
			// we had a shitty hand, just to annoy the player
			if( Math.random() < 0.5 || this.m_betRound == BetRound.PRE_FLOP ) // we only show our cards half the time, and never pre-flop
				return false;
				
			// if we had a better hand than the player, then tell them, otherwise taunt
			var resultType:ResultType = this.m_aiPlayer.cardMatchResults.compareAgainst( this.m_mainPlayer.cardMatchResults );
			if( resultType == ResultType.WIN && this.m_aiPlayer.cardMatchResults.type != HandRank.HIGH_CARD )
				this.m_aiPlayer.say( "Good fold!" );
			else
				this.m_aiPlayer.say( "Lol!" );
			return true;
		}
		
		/**********************************************************************/
		
		// folds our cards
		private function _fold():void
		{
			this.m_aiPlayer.say( "Fold" );
			this.onFold();
		}
		
		// doesn't bet - checks
		private function _check():void
		{
			this.m_aiPlayer.say( "Check" );
			this.onCheck();
		}
		
		// calls the bet
		private function _call( amount:int ):void
		{
			// when calling, we might be all-in, but under the opponent bet
			if ( amount >= this.m_aiPlayer.money )
			{
				amount = this.m_aiPlayer.money;
				this.m_aiPlayer.say( "Call, all-in!" );
			}
			else
				this.m_aiPlayer.say( "Call!" );
			this.onCall( amount );
		}
		
		// raises the bet
		private function _raise( amount:int ):void
		{
			// check that we're not raising over our limit
			if ( amount >= this.m_aiPlayer.money )
				amount = this.m_aiPlayer.money;
				
			// if our amount doesn't cover the player bet, then switch to call (we're all in)
			if( amount <= this.m_opponentBet )
			{
				this._call( amount );
				return;
			}
			
			// raise it up
			if( amount == this.m_aiPlayer.money )
				this.m_aiPlayer.say( "Raise, all-in!" );
			else
				this.m_aiPlayer.say( "Raise, " + amount + "!" );
			this.onRaise( amount );
		}
		
	}
}