package logic
{
	import System.XML.XMLNode;
	import System.XML.XMLElement;
	import System.XML.XMLAttribute;
	
	/**
	 * Define a delegate so we can add default callbacks for the ai player
	 */
	public delegate AIPlayerCallback():void;
	
	/**
	 * Define a delegate so we can add callbacks for when the ai player talks
	 */
	public delegate AIPlayerSpeechCallback( msg:String, autoHide:Boolean ):void;
	
	/**
	 * Define a delegate so we can add callbacks for animations
	 */
	public delegate AIPlayerAnimCallback( anim:String ):void;
	
	/**
	 * An AIPlayer is the base class for a person (main or ai) in a match
	 */
	public class AIPlayer extends Player
	{
		
		/**********************************************************************/
		
		/**
		 * Called when the ai player wants to say something. It should take one parameter of 
		 * type String (msg) and one of type Boolean (autoHide)
		 */
		public var onTalk:AIPlayerSpeechCallback = null;
		
		/**
		 * Called when the ai player wants to stop talking. It should take no parameters
		 */
		public var onStopTalking:AIPlayerCallback = null;
		
		/**
		 * Called when we want to play an animation. It should take one parameter of type String
		 */
		public var onPlayAnim:AIPlayerAnimCallback = null;
		
		/** 
		 * The AI for this player
		 */
		public var ai:AI = null;
		
		/**
		 * The name of the player
		 */
		public var name:String = null;
		
		/**
		 * The buy-in to play this player, and the amount of money we'll start with
		 */
		public var buyIn:int = 0;
		
		/**
		 * The small blind for the match - doesn't change
		 */
		public var smallBlind:int = 0;
		
		/**
		 * The big blind for the match - doesn't change
		 */
		public var bigBlind:int = 0;
		
		/**
		 * How aggressive we are
		 */
		public var aggressiveness:Number = 0.0;
		
		/**
		 * How intelligent we are
		 */
		public var intelligence:Number = 0.0;
		
		/**
		 * The name of our graphics for the menu
		 */
		public var menuBtnGraphics:String = null;
		
		/**
		 * The name of the down graphics for the menu
		 */
		public var menuBtnGraphicsDown:String = null;
		
		/**
		 * The name of the disabled graphics for the menu
		 */
		public var menuBtnGraphicsDisabled:String = null;
		
		/**
		 * The name of the player graphics, for in the play state
		 */
		public var playerGraphics:String = null;
		
		/**********************************************************************/
		
		/**
		 * Get the AI player to say something
		 * @param msg The message to say
		 * @param autoHide Should we auto hide the message?
		 */
		public function say( msg:String, autoHide:Boolean = true ):void
		{
			this.onTalk( msg, autoHide );
		}
		
		/**
		 * Get the AI player to stop talking
		 */
		public function shutup():void
		{
			this.onStopTalking();
		}
		
		/**
		 * Reveal the hole cards that we have
		 */
		public function showHoleCards():void
		{
			// check that we have them
			if( this.holeCard1 == null || this.holeCard2 == null )
			{
				Console.print( "[AIPlayer] ERROR: Can't show our hole cards as they're not set" );
				return;
			}
			
			// reveal them
			this.holeCard1.reveal();
			this.holeCard2.reveal();
		}
		
		/**
		 * Plays an anim on the ai player
		 * @param anim The anim to play; use the Animation constants
		 */
		public function playAnim( anim:String ):void
		{
			this.onPlayAnim( anim );
		}
		
		/**
		 * Reads our data from XML
		 * @param x The XMLNode that contains our data
		 */
		public function fromXML( x:XMLNode ):void
		{
			// get the name xml
			var nx:XMLNode = x.firstChild();
			if( nx == null )
			{
				Console.print( "[AIPlayer] ERROR: Can't find the name node" );
				return;
			}
			else
			{
				var nxn:XMLNode	= nx.firstChild(); // nx is <name>someName</name> here, we need to get it's first child to get "someName"
				this.name 		= ( nxn != null ) ? nxn.getValue() : null;
			}
				
			// check it
			if( this.name == null || this.name == "null" || this.name == "" )
				Console.print( "[AIPlayer] ERROR: Can't find the player's name from node " + nx );
			
			// get the game xml
			var gx:XMLNode = nx.nextSibling();
			if( gx == null )
			{
				Console.print( "[AIPlayer] ERROR: Can't find the game node for player '" + this.name + "'" );
				return;
			}
			
			// convert it to an element and find our attributes
			var gxe:XMLElement 		= gx.toElement();
			var ba:XMLAttribute		= ( gxe != null ) ? gxe.findAttribute( "buyin" ) : null;
			var sba:XMLAttribute 	= ( gxe != null ) ? gxe.findAttribute( "small" ) : null;
			var bba:XMLAttribute	= ( gxe != null ) ? gxe.findAttribute( "big" ) : null;
			var aa:XMLAttribute		= ( gxe != null ) ? gxe.findAttribute( "aggressiveness" ) : null;
			var ia:XMLAttribute		= ( gxe != null ) ? gxe.findAttribute( "intelligence" ) : null;
			
			// set our properties
			this.buyIn 			= ( ba != null ) ? ba.numberValue : 0;
			this.smallBlind		= ( sba != null ) ? sba.numberValue : 0;
			this.bigBlind		= ( bba != null ) ? bba.numberValue : 0;
			this.aggressiveness	= ( aa != null ) ? aa.numberValue : 0;
			this.intelligence	= ( ia != null ) ? ia.numberValue : 0;
			if( this.buyIn == 0 || this.smallBlind == 0 || this.bigBlind == 0 || this.aggressiveness == 0 || this.intelligence == 0 )
				Console.print( "[AIPlayer] ERROR: Couldn't read the buyIn (" + this.buyIn + "), small blind (" + this.smallBlind + ") or big blind (" + this.bigBlind + "), aggressiveness (" + this.aggressiveness + "), intelligence (" + 
				this.intelligence + ") for the player '" + this.name + "'" );
				
			// get the menu node
			var mx:XMLNode = gx.nextSibling();
			if( mx == null )
			{
				Console.print( "[AIPlayer] ERROR: Can't find the menu node for player '" + this.name + "'" );
				return;
			}
			
			// convert it to an element and find our attribute
			var mxe:XMLElement 	= mx.toElement();
			var ga:XMLAttribute	= ( mxe != null ) ? mxe.findAttribute( "graphics" ) : null;
			
			// set our properties
			var graphicsName:String = ( ga != null ) ? ga.value : null;
			if( graphicsName == null )
				Console.print( "[AIPlayer] ERROR: Couldn't read the menu graphics name for the player '" + this.name + "'" );
			else
			{
				this.menuBtnGraphics 			= graphicsName;
				this.menuBtnGraphicsDown		= graphicsName + "_down";
				this.menuBtnGraphicsDisabled	= graphicsName + "_disabled";
			}
			
			// get the play node
			var px:XMLNode = mx.nextSibling();
			if( px == null )
			{
				Console.print( "[AIPlayer] ERROR: Can't find the play node for player '" + this.name + "'" );
				return;
			}
			
			// convert it to an element and find our attribute
			var pxe:XMLElement 	= px.toElement();
			var pa:XMLAttribute	= ( pxe != null ) ? pxe.findAttribute( "graphics" ) : null;
			
			// set our property
			this.playerGraphics = ( pa != null ) ? pa.value : null;
			if( this.playerGraphics == null )
				Console.print( "[AIPlayer] ERROR: Couldn't read the play graphics name for the player '" + this.name + "'" );
		}
		
		/**
		 * The String version of the player
		 */
		public function toString():String
		{
			return "[AIPlayer name: " + this.name + ", money: " + this.money + ", buyIn: " + this.buyIn + ", blinds: " + this.smallBlind + "/" + this.bigBlind + ", aggressiveness: " + this.aggressiveness + ", intelligence: " + this.intelligence + "]";
		}
		
	}
}