package logic
{
	import enums.BetRound;
	import enums.HandRank;
	import state.PokerGameState;
	
	/**
	 * Define a delegate so we can add callbacks to the bet logic
	 */
	public delegate BetLogicCallback():void;
	
	/**
	 * Controls the logic for betting in a poker state
	 */
	public class BetLogic
	{
		
		/**********************************************************************/
		
		/**
		 * Called when we're finished and we can move on. It should take no parameters
		 */
		public var onFinish:BetLogicCallback = null;
		
		/**
		 * The Player that's folding
		 */
		public var foldingPlayer:Player = null;
		
		/**********************************************************************/
		
		private var m_state:PokerGameState 		= null; // the state that this is for
		private var m_playerIsDealer:Boolean	= false;// is the player the dealer?
		private var m_hasDealerSpoken:Boolean	= false;// has the dealer spoken in pre-flop?
		private var m_playerCurrBet:int			= 0;	// the player's current bet total
		private var m_opponentCurrBet:int		= 0;	// the opponent's current bet total
		private var m_smallBlind:int			= 0;	// the small blind
		private var m_bigBlind:int				= 0;	// the big blind
		private var m_betRound:BetRound			= BetRound.NONE; // the current round of betting that we're in
		
		/**********************************************************************/
		
		/**
		 * Creates a new BetLogic class
		 * @param state The PokerGameState that this is for
		 */
		public function BetLogic( state:PokerGameState )
		{
			this.m_state = state;
		}
		
		/**
		 * Destroys the BetLogic and clears it for garbage collection
		 */
		public function destroy():void
		{
			// clear our properties
			this.onFinish		= null;
			this.foldingPlayer	= null;
			this.m_state		= null;
		}
		
		/**
		 * Begins the bet logic
		 * @param round The current betting round that we're in
		 */
		public function begin( round:BetRound ):void
		{
			// reset our vars
			this.foldingPlayer 		= null;
			this.m_playerCurrBet	= 0;
			this.m_opponentCurrBet	= 0;
			this.m_playerIsDealer	= this.m_state.mainPlayer.isDealer;
			this.m_smallBlind		= this.m_state.opponent.smallBlind;
			this.m_bigBlind			= this.m_state.opponent.bigBlind;
			this.m_betRound			= round;
			this.m_hasDealerSpoken	= false;
			
			// add our delegate listeners
			this.m_state.opponent.ai.onCheck	+= this._onOpponentCheck;
			this.m_state.opponent.ai.onCall		+= this._onOpponentCall;
			this.m_state.opponent.ai.onRaise	+= this._onOpponentRaise;
			this.m_state.opponent.ai.onFold		+= this._onOpponentFold;
			
			// if we should pay the blinds do so
			if( this.m_betRound == BetRound.PRE_FLOP )
			{
				// get the blinds
				this.m_playerCurrBet 	= ( this.m_playerIsDealer ) ? this.m_bigBlind : this.m_smallBlind;
				this.m_opponentCurrBet	= ( this.m_playerIsDealer ) ? this.m_smallBlind : this.m_bigBlind;
				
				// check if it puts one of the players all-in
				if( this.m_playerCurrBet > this.m_state.mainPlayer.money )
					this.m_playerCurrBet = this.m_state.mainPlayer.money;
				if( this.m_opponentCurrBet > this.m_state.opponent.money )
					this.m_opponentCurrBet = this.m_state.opponent.money;
					
				// pay the blinds
				this.m_state.mainPlayer.payCash( this.m_playerCurrBet );
				this.m_state.opponent.payCash( this.m_opponentCurrBet );
			}
				
			// if the player is the dealer, then they're the big blind, so it's up to the ai to speak first.
			// otherwise the player is the small blind, and they need to call/raise the rest of the big blind/go second
			// NOTE: check if the blinds put them all in
			if( this.m_playerIsDealer && this.m_state.opponent.money > 0 )
				this.m_state.opponent.ai.think( this.m_opponentCurrBet, this.m_playerCurrBet, this.m_betRound ); // ai bet, player bet
			else if( !this.m_playerIsDealer && this.m_state.mainPlayer.money > 0 )
				this._showActionButtons();
			else
				this.onFinish();
		}
		
		/**
		 * Ends the bet logic
		 */
		public function end():void
		{
			// remove our delegate listeners
			this._hideActionButtons();
			this.m_state.opponent.ai.onCheck	-= this._onOpponentCheck;
			this.m_state.opponent.ai.onCall		-= this._onOpponentCall;
			this.m_state.opponent.ai.onRaise	-= this._onOpponentRaise;
			this.m_state.opponent.ai.onFold		-= this._onOpponentFold;
		}
		
		/**********************************************************************/
		
		// shows the action buttons for the player
		private function _showActionButtons():void
		{
			// add our delegate listeners
			this.m_state.actionButtons.onCheck 	+= this._onPlayerCheck;
			this.m_state.actionButtons.onCall	+= this._onPlayerCall;
			this.m_state.actionButtons.onRaise	+= this._onPlayerRaise;
			this.m_state.actionButtons.onFold	+= this._onPlayerFold;
			
			// show the buttons
			this.m_state.actionButtons.show( this.m_opponentCurrBet, this.m_playerCurrBet, ( this.m_state.opponent.money <= 0 ) ); // ai bet, already paid, is opponent all-in
		}
		
		// hides the action buttons for the player
		private function _hideActionButtons():void
		{
			// remove our delegate listeners (stops double clicks when leaving)
			this.m_state.actionButtons.onCheck 	-= this._onPlayerCheck;
			this.m_state.actionButtons.onCall	-= this._onPlayerCall;
			this.m_state.actionButtons.onRaise	-= this._onPlayerRaise;
			this.m_state.actionButtons.onFold	-= this._onPlayerFold;
			
			// hide the buttons
			this.m_state.actionButtons.hide();
		}
		
		// the player is checking
		private function _onPlayerCheck():void
		{
			Console.print( "[BetLogic] Player Check!" );
			
			// we need to check if the player is the dealer. if so, then we move on, otherwise, it's
			// the ai to speak (provided they have money to do so)
			this._hideActionButtons();
			if( this.m_playerIsDealer || this.m_state.opponent.money <= 0 )
				this.onFinish();
			else
				this.m_state.opponent.ai.think( this.m_opponentCurrBet, this.m_playerCurrBet, this.m_betRound ); // ai bet, player bet
		}
		
		// the player is calling
		private function _onPlayerCall( amount:int ):void
		{
			Console.print( "[BetLogic] Player call: " + amount );
			// if the player is calling, it can mean 2 things: it's a normal round, and they're
			// calling a bet from the opponent, in which case we move on. Or it's pre-flop and
			// they're the small blind, in which case the ai has another chance to check or raise.
			// we also check if the dealer has spoken, otherwise the player could call, the opponent
			// raise, the player call again, and the opponent would get another chance
			
			// get the player to pay what they've called, minus what they've already paid.
			// if the amount is equal to the amount of money that the player has, then they're all in
			if( amount == this.m_state.mainPlayer.money )
			{
				this.m_state.mainPlayer.payCash( amount );
				this.m_state.mainPlayer.isAllIn = true;
				this.m_playerCurrBet 			+= amount;
				
				// if the player's bet is less than the opponent's bet, return the difference to the opponent
				if( this.m_playerCurrBet < this.m_opponentCurrBet )
				{
					this.m_state.opponent.winCash( this.m_opponentCurrBet - this.m_playerCurrBet );
					this.m_opponentCurrBet = this.m_playerCurrBet;
				}
			}
			else
			{
				this.m_state.mainPlayer.payCash( amount - this.m_playerCurrBet );
				this.m_playerCurrBet = amount;
			}
			this._hideActionButtons();
			
			// if we're pre-flop and the dealer, then we've spoken - this stops a bug where the player is the dealer, so the
			// opponent is first to speak. He raises, we raise, he calls - we would then get another go, which is wrong
			if( this.m_betRound == BetRound.PRE_FLOP && this.m_playerIsDealer )
				this.m_hasDealerSpoken = true;
			
			// if we're pre-flop and the player isn't the dealer, and the opponent still has
			// some cash, then give the ai a chance, otherwise move on
			if( this.m_betRound == BetRound.PRE_FLOP && !this.m_playerIsDealer && !this.m_hasDealerSpoken && this.m_state.opponent.money > 0 )
			{
				this.m_hasDealerSpoken = true;
				this.m_state.opponent.ai.think( this.m_opponentCurrBet, this.m_playerCurrBet, this.m_betRound ); // ai bet, player bet
			}
			else
				this.onFinish();
		}
		
		// the player is raising
		private function _onPlayerRaise( amount:int ):void
		{
			Console.print( "[BetLogic] Player raise: " + amount + ( ( amount == this.m_state.mainPlayer.money ) ? " - all in!" : "" ) );
			
			// the player is raising, so regardless of the round we're in, the ai gets another go.
			// if the amount is equal to the amount of money that the player has, then they're all in
			if( amount == this.m_state.mainPlayer.money )
			{
				this.m_state.mainPlayer.payCash( amount );
				this.m_state.mainPlayer.isAllIn = true;
				this.m_playerCurrBet 			+= amount;
			}
			else
			{
				// the player is raising to amount - pay what's left to pay (we might have already bet this round)
				this.m_state.mainPlayer.payCash( amount - this.m_playerCurrBet );
				this.m_playerCurrBet = amount;
			}
			this._hideActionButtons();
			
			// if we're pre-flop and the dealer, then we've spoken - this stops a bug where the player is the dealer, so the
			// opponent is first to speak. He raises, we raise, he calls - we would then get another go, which is wrong
			if( this.m_betRound == BetRound.PRE_FLOP && this.m_playerIsDealer )
				this.m_hasDealerSpoken = true;
			
			// if the player doesn't have anything in their hand (post flop), count it as a bluff
			if( this.m_betRound != BetRound.PRE_FLOP && ( this.m_state.mainPlayer.cardMatchResults.type == HandRank.HIGH_CARD ) )
				this.m_state.mainPlayer.statsNumBluffs += 1;
			
			// tell our ai (normally we don't need this check, but to be safe)
			if( this.m_state.opponent.money > 0 )
				this.m_state.opponent.ai.think( this.m_opponentCurrBet, this.m_playerCurrBet, this.m_betRound ); // ai bet, player bet
			else
				this.onFinish();
		}
		
		// the player is folding
		private function _onPlayerFold():void
		{
			Console.print( "[BetLogic] Player folding" );
			this.foldingPlayer = this.m_state.mainPlayer;
			this._hideActionButtons();
			this.onFinish();
		}
		
		// the opponent is checking
		private function _onOpponentCheck():void
		{
			Console.print( "[BetLogic] Opponent Check!" );
			// we need to check to see if the opponent is the dealer. if so, then we move on, otherwise
			// the player has a go (provided they can)
			if( !this.m_playerIsDealer || this.m_state.mainPlayer.money <= 0 )
				this.onFinish();
			else
				this._showActionButtons();
		}
		
		// the opponent is calling
		private function _onOpponentCall( amount:int ):void
		{
			Console.print( "[BetLogic] Opponent call: " + amount );
			// if the opponent is calling, it can mean 2 things: it's a normal round, and they're
			// calling a bet from the player, in which case we move on. Or it's pre-flop and
			// they're the small blind, in which case the player has another chance to check or raise.
			// we also check if the dealer has spoken, otherwise the opponent could call, the player
			// raise, the opponent call again, and the player would get another chance
			
			// get the opponent to pay what they've called, minus what they've already paid.
			// if the amount is equal to the amount of money that the opponent has left, then they're all in
			if( amount == this.m_state.opponent.money )
			{
				this.m_state.opponent.payCash( amount );
				this.m_state.opponent.isAllIn 	= true;
				this.m_opponentCurrBet 			+= amount;
				
				// if the opponent's bet is less than the player's bet, return the difference to the player
				if( this.m_opponentCurrBet < this.m_playerCurrBet )
				{
					this.m_state.mainPlayer.winCash( this.m_playerCurrBet - this.m_opponentCurrBet );
					this.m_playerCurrBet = this.m_opponentCurrBet;
				}
			}
			else
			{
				this.m_state.opponent.payCash( amount - this.m_opponentCurrBet );
				this.m_opponentCurrBet = amount;
			}
			
			// if we're pre-flop and the dealer, then we've spoken - this stops a bug where the opponent is the dealer, so the
			// player is first to speak. They raise, we raise, they calls - we would then get another go, which is wrong
			if( this.m_betRound == BetRound.PRE_FLOP && !this.m_playerIsDealer )
				this.m_hasDealerSpoken = true;
			
			// if we're pre-flop and the opponent isn't the dealer, and the player has money left
			// then give the player a chance, otherwise move on
			if( this.m_betRound == BetRound.PRE_FLOP && this.m_playerIsDealer && !this.m_hasDealerSpoken && this.m_state.mainPlayer.money > 0 )
			{
				this.m_hasDealerSpoken = true;
				this._showActionButtons();
			}
			else
				this.onFinish();
		}
		
		// the opponent is raising
		private function _onOpponentRaise( amount:int ):void
		{
			Console.print( "[BetLogic] Opponent raise: " + amount + ( ( amount == this.m_state.opponent.money ) ? " - all in!" : "" ) );
			
			// the opponent is raising, so the player gets another go.
			// if the amount is equal to the amount of money that the opponent has left, then they're all in
			if( amount == this.m_state.opponent.money )
			{
				this.m_state.opponent.payCash( amount );
				this.m_state.opponent.isAllIn 	= true;
				this.m_opponentCurrBet 			+= amount;
			}
			else
			{
				// the opponent is raising to amount - pay what's left to pay (we might have already bet this round)
				this.m_state.opponent.payCash( amount - this.m_opponentCurrBet );
				this.m_opponentCurrBet = amount;
			}
			
			// if we're pre-flop and the dealer, then we've spoken - this stops a bug where the opponent is the dealer, so the
			// player is first to speak. They raise, we raise, they calls - we would then get another go, which is wrong
			if( this.m_betRound == BetRound.PRE_FLOP && !this.m_playerIsDealer )
				this.m_hasDealerSpoken = true;
			
			// show our action buttons (normally we shouldn't need this check, but to be safe)
			if( this.m_state.mainPlayer.money > 0 )
				this._showActionButtons();
			else
				this.onFinish();
		}
		
		// the opponent is folding
		private function _onOpponentFold():void
		{
			// if the opponent folded, and the player has bet something, and they have nothing in their hand
			// then we count it as a bluff won - NOTE: ignore blinds
			if( ( this.m_betRound != BetRound.PRE_FLOP && this.m_playerCurrBet > 0 ) || 
				( this.m_betRound == BetRound.PRE_FLOP && this.m_playerCurrBet > this.m_bigBlind ) )
			{
				if( this.m_state.mainPlayer.cardMatchResults.type == HandRank.HIGH_CARD )
					this.m_state.mainPlayer.statsNumBluffsWon += 1;
			}
				
			// finish
			Console.print( "[BetLogic] Opponent folding" );
			this.foldingPlayer = this.m_state.opponent;
			this.onFinish();
		}
		
	}
}