package logic
{	
	import enums.HandRank;
	import enums.ResultType;
	
	/**
	 * The match results that we return when we do a match on a set of cards
	 */
	public class CardMatchResults
	{
		
		/*******************************************************************************************/
		
		/**
		 * The type of this result
		 */
		public var type:HandRank = HandRank.HIGH_CARD;
		
		/**
		 * Was it a successful match?
		 */
		public var success:Boolean = false;
		
		/** 
		 * The best hand we can make
		 */
		public var bestHand:Vector.<Card> = null;
		
		/*******************************************************************************************/
		
		/**
		 * Creates a new CardMatchResults object
		 */
		public function CardMatchResults()
		{
			this.bestHand = new Vector.<Card>;
		}
		
		/**
		 * Resets the MatchResults so it can be reused
		 */
		public function reset():void
		{
			this.success	= false;
			this.type 		= HandRank.HIGH_CARD;
			this.bestHand.clear();
		}
		
		/**
		 * Destroys the CardMatchResult and clears it for garbage collection
		 */
		public function destroy():void
		{
			this.type		= null;
			this.bestHand	= null;
		}
		
		/**
		 * Are these card match results better than other results?
		 * @return A ResultType that says if we're going to win, lose or draw
		 */
		public function compareAgainst( other:CardMatchResults ):ResultType
		{
			// quick check against the type
			if( this.type > other.type )
				return ResultType.WIN;
			if( this.type < other.type )
				return ResultType.LOSE;
		
			// our types are the same, check against the cards
			if( this.type == HandRank.ROYAL_FLUSH )
				return ResultType.SPLIT_POT; // same hand
			else
			{
				// go through the best hands and see if they match
				var len:int = this.bestHand.length;
				for( var i:int = 0; i < len; i++ )
				{
					if( this.bestHand[i].value > other.bestHand[i].value )
						return ResultType.WIN;
					else if( this.bestHand[i].value < other.bestHand[i].value )
						return ResultType.LOSE;
				}
			}
			
			// it's a split pot
			return ResultType.SPLIT_POT;
		}
		
		/**
		 * Gets the win string for this card match results
		 */
		public function getWinString():String
		{
			// we need at least 5 cards
			if( this.bestHand == null || this.bestHand.length != 5 )
			{
				Console.print( "[CardMatchResults] ERROR: Can't get the win string, as we don't have enough cards in our best hand" );
				return "";
			}
			
			// our cards as a convenience param
			var card1:Card = this.bestHand[0];
			var card2:Card = this.bestHand[1];
			var card3:Card = this.bestHand[2];
			var card4:Card = this.bestHand[3];
			var card5:Card = this.bestHand[4];
			
			// depending on our result, return the string
			// the string is in the form "you with with..." or "opponent wins with..."
			if( type == HandRank.HIGH_CARD )
				return "a high card: " + card1.valueString + " of " + card1.suitString + "!";
			else if( type == HandRank.PAIR )
				return "a pair of " + card1.valuePluralString + "!";
			else if( type == HandRank.TWO_PAIR )
				return "two pair: " + card1.valuePluralString + " and " + card3.valuePluralString + "!";
			else if( type == HandRank.TRIPS )
				return "trip " + card1.valuePluralString + "!";
			else if( type == HandRank.STRAIGHT )
				return "a straight, " + card1.valueString + " high!";
			else if( type == HandRank.FLUSH )
				return "a flush of " + card1.suitString + ", " + card1.valueString + " high!";
			else if( type == HandRank.FULL_HOUSE )
				return "a full house; " + card1.valuePluralString + " over " + card4.valuePluralString + "!";
			else if( type == HandRank.POKER )
				return "a poker of " + card1.valuePluralString + "!";
			else if( type == HandRank.STRAIGHT_FLUSH )
				return "a straight flush: " + card1.suitString + ", " + card1.valueString + " high!";
			else if( type == HandRank.ROYAL_FLUSH )
				return "a royal flush of " + card1.suitString + "!";
				
			// the hand rank was wrong
			Console.print( "[CardMatchResults] ERROR: Can't get the win string, as type (" + this.type + ") isn't good" );
			return "";
		}
		
		/**
		 * The string version of the MatchResults
		 */
		public function toString():String
		{
			return "[CardMatchResults type: " + this._getRankString( this.type ) + ", success: " + this.success + 
				", bestHand (" + this.bestHand.length + "): " + this.bestHand + "]";
		}
		
		/*******************************************************************************************/
		
		// returns a HandRank enum as a string
		private function _getRankString( rank:HandRank ):String
		{
			if( rank == HandRank.HIGH_CARD ) return "High card";
			else if( rank == HandRank.PAIR ) return "Pair";
			else if( rank == HandRank.TWO_PAIR ) return "Two pair";
			else if( rank == HandRank.TRIPS ) return "Trips";
			else if( rank == HandRank.STRAIGHT ) return "Straight";
			else if( rank == HandRank.FLUSH ) return "Flush";
			else if( rank == HandRank.FULL_HOUSE ) return "Full house";
			else if( rank == HandRank.POKER ) return "Poker";
			else if( rank == HandRank.STRAIGHT_FLUSH ) return "Straight flush";
			else if( rank == HandRank.ROYAL_FLUSH ) return "Royal flush";
			
			// couldn't find it
			Console.print( "[CardMatchResults] ERROR: Unknown HandRank '" + rank + "' when getting the string" );
			return "";
		}
		
	}

}