package logic
{
	import enums.CardDealDest;
	import enums.CardValue;
	import enums.CardSuit;
	
	/**
	 * Define a delegate so we can add callbacks to the deck
	 */
	public delegate DeckCallback( deck:Deck ):void;
	
	/**
	 * Define a delegate so we can add callbacks to the deck and card
	 */
	public delegate DeckCardCallback( deck:Deck, card:Card, dest:CardDealDest, delay:Number ):void;
	
	/**
	 * Define a delegate so we can add callbacks to the deck and hand
	 */
	public delegate DeckHandCallback( deck:Deck, cards:Vector.<Card> ):void;
	
	/**
	 * A Deck of Card objects
	 */
	public class Deck
	{
		
		/**********************************************************************/
		
		/**
		 * The callback called when we shuffle the deck. It should take one parameter of type Deck
		 */
		public var onShuffle:DeckCallback = null;
		
		/**
		 * The callback called when we burn a card. It should take one parameter of type Deck
		 */
		public var onBurn:DeckCallback = null;
		
		/**
		 * The callback called when we deal a card. It should take one parameter of type Deck and
		 * one of type Card, and on of type Number (delay)
		 */
		public var onDeal:DeckCardCallback = null;
		
		/**
		 * The callback called when we want to gather up all the cards. It should take one parameter
		 * of type Deck
		 */
		public var onGatherCards:DeckCallback = null;
		
		/**
		 * Called when we want to show the winning hand to the player. It should take one parameter
		 * of type Deck and one of type Vector.<Card>
		 */
		public var onShowWinningHand:DeckHandCallback = null;
		
		/**
		 * The vector of cards currently on the table
		 */
		public var tableCards:Vector.<Card> = null;
		
		/**********************************************************************/
		
		private var m_cards:Vector.<Card> 	= null; // our vector of cards
		private var m_index:int				= 0;	// when picking a card, this is the index
		
		/**********************************************************************/
		
		/**
		 * The number of cards in this deck
		 */
		public function get numCards():int { return this.m_cards.length; }
		
		/**********************************************************************/
		
		/**
		 * Creates a new Deck of cards
		 */
		public function Deck()
		{
			// create our cards
			var len:int 	= 52;
			this.m_cards	= new Vector.<Card>;
			for( var i:int = 0; i < len; i++ )
			{
				var value:CardValue = ( i % 13 );
				var suit:CardSuit	= int( i / 13 );
				var card:Card		= new Card( value, suit );
				this.m_cards.push( card );
			}
			
			// create our table cards vector
			this.tableCards = new Vector.<Card>();
		}
		
		/**
		 * Destroys the Deck and clears it for garbage collection
		 */
		public function destroy():void
		{
			// destroy our cards
			var len:int = this.m_cards.length;
			for( var i:int = 0; i < len; i++ )
				this.m_cards[i].destroy();
				
			// null our properties
			this.onShuffle			= null;
			this.onBurn				= null;
			this.onDeal				= null;
			this.onGatherCards		= null;
			this.onShowWinningHand	= null;
			this.tableCards			= null;
			this.m_cards 			= null;
		}
		
		/**
		 * Shuffles the cards
		 */
		public function shuffle():void
		{
			// reset our index and clear our hole cards
			this.m_index = 0;
			
			// shuffle our cards
			var len:int = this.m_cards.length;
			for( var i:int = 0; i < len; i++ )
			{
				var card:Card 	= this.m_cards[i];
				card.dest		= null; // clear our dest
				var ri:int		= int(Math.random() * len); // random index
				
				// swap the cards
				this.m_cards[i] 	= this.m_cards[ri];
				this.m_cards[ri]	= card;
			}
			
			// clear our table cards vector
			this.tableCards.clear();
			
			// call our delegate
			this.onShuffle( this );
		}
		
		/**
		 * Burns a card
		 */
		public function burn():void
		{
			this._incretIndex();
			
			// call our delegate
			this.onBurn( this );
		}
		
		/**
		 * Deals a card from the deck
		 * @param dest The destination for this card
		 * @param delay The delay for dealing this card
		 * @return The Card that we dealt
		 */
		public function deal( dest:CardDealDest, delay:Number = 0.0 ):Card
		{
			var card:Card 	= this.m_cards[this.m_index];
			card.dest		= dest; // hold our destination
			this._incretIndex();
			
			// if this is a table card, store it
			if( dest >= CardDealDest.FLOP_1 )
				this.tableCards.push( card );
			
			// call our delegate
			this.onDeal( this, card, dest, delay );
			return card;
		}
		
		/**
		 * Gathers up all the cards on the table and returns them to the deck
		 */
		public function gatherCards():void
		{
			this.onGatherCards( this );
		}
		
		/**
		 * Shows the winning hand to the player
		 * @param cards The Vector of CardDealDest for the winning hand
		 */
		public function showWinningHand( cards:Vector.<Card> ):void
		{
			// make sure our hand is good
			if( cards == null || cards.length != 5 )
			{
				Console.print( "[Deck] ERROR: Can't show the winning hand as our cards vector is wrong" );
				return;
			}
			this.onShowWinningHand( this, cards );
		}
		
		/**********************************************************************/
		
		// incretements our index
		private function _incretIndex():void
		{
			this.m_index++;
			
			// failsafe
			if( this.m_index >= this.m_cards.length )
				this.m_index = 0;
		}
		
	}
}