package logic
{	

	/**
	 * Define a delegate so we can add callbacks to the player, passing this
	 */
	public delegate PlayerCallback( player:Player ):void;
	
	/**
	 * Define a delegate so we can add callbacks to the player with an amount
	 */
	public delegate PlayerAmountCallback( amount:int ):void;
	
	/**
	 * Define a delegate so we can add callbacks to the player with the results, and if the opponent folded/if it was a split pot
	 */
	public delegate PlayerResultCallback( results:CardMatchResults, opponentFolded:Boolean, splitPot:Boolean ):void;
	
	/**
	 * A Player is the base class for a person (main or ai) in a match
	 */
	public class Player
	{
		
		/**********************************************************************/
		
		/**
		 * The callback dispatched when we become dealer. It should take one parameter
		 * of type Player
		 */
		public var onIsDealer:PlayerCallback = null;
		
		/**
		 * The callback dispatched when we pay some cash. It should take one parameter
		 * of type int
		 */
		public var onPayCash:PlayerAmountCallback = null;
		
		/**
		 * The callback dispatched when we win some cash. It should take one parameter
		 * of type int
		 */
		public var onWinCash:PlayerAmountCallback = null;
		
		/**
		 * The callback dispatched when we win The hand. It should take one parameter
		 * of type CardMatchResults, and two of type Boolean (if the opponent folded, split pot)
		 */
		public var onWinHand:PlayerResultCallback = null;
		
		/**
		 * Our first hole card
		 */
		public var holeCard1:Card = null;
		
		/**
		 * Our second hole card
		 */
		public var holeCard2:Card = null;
		
		/**
		 * Our card match results
		 */
		public var cardMatchResults:CardMatchResults = null;
		
		/**
		 * The amount of money that we're bringing into this match
		 */
		public var money:int = 0;
		
		/**********************************************************************/
		
		protected var m_isDealer:Boolean 	= false; // are we the dealer?
		protected var m_isAllIn:boolean		= false;// are we all-in?
		
		/**********************************************************************/
		
		/**
		 * Are we the dealer?
		 */
		public function get isDealer():Boolean { return this.m_isDealer; }
		public function set isDealer( b:Boolean ):void
		{
			this.m_isDealer = b;
			if( this.m_isDealer )
				this.onIsDealer( this );
		}
		
		/**
		 * Are we all-in?
		 */
		public function get isAllIn():Boolean { return this.m_isAllIn; }
		public function set isAllIn( b:Boolean ):void
		{
			this.m_isAllIn = b;
		}
		
		/**********************************************************************/
		
		/**
		 * Creates a new Player
		 */
		public function Player()
		{
			this.cardMatchResults = new CardMatchResults();
		}
		
		/**
		 * Destroys the Player and clears it for garbage collection
		 */
		public function destroy():void
		{
			// destroy our match results
			this.cardMatchResults.destroy();
			
			// clear our properties
			this.onPayCash 			= null;
			this.onWinCash 			= null;
			this.holeCard1			= null;
			this.holeCard2			= null;
			this.cardMatchResults	= null;
		}
		
		/**
		 * Pay some cash
		 * @param amount The amount of cash that we have to pay
		 * @return The amount that we paid
		 */
		public function payCash( amount:int ):int
		{
			// if we're not paying anything, do nothing
			if( amount <= 0 )
				return 0;
				
			// check our amount
			if( amount > this.money )
			{
				Console.print( "[Player] ERROR: Being told to pay " + amount + " cash, but we only have " + this.money + ", changing it" );
				amount = this.money;
			}
			
			// pay it
			this.money -= amount;
			this.onPayCash( amount );
			return amount;
		}
		
		/**
		 * The player wins the hand
		 * @param opponentFolded Did our opponent fold?
		 * @param splitPot Was this a split pot?
		 */
		public function winHand( opponentFolded:Boolean, splitPot:Boolean ):void
		{
			this.onWinHand( this.cardMatchResults, opponentFolded, splitPot );
		}
		
		/** 
		 * Win some cash
		 * @param amount The amount of cash that we just won
		 */
		public function winCash( amount:int ):void
		{
			// check our amount
			if( amount <= 0 )
			{
				Console.print( "[Player] ERROR: Being told to win " + amount + " cash" );
				return;
			}
			
			// add it and call our delegate
			this.money += amount;
			this.onWinCash( amount );
		}
		
	}
}