package logic.animations
{
	import constants.Animations;
	import gui.PlayerGraphics;
	import logic.AIPlayer;
	
	/**
	 * The animation controller, that controls all the anims for our player
	 */
	public class AnimController
	{
		
		/**********************************************************************/
		
		private var m_anims:Vector.<Animation> 	= null; // the vector of our animations
		private var m_currAnim:Animation		= null;	// our current animation
		private var m_graphics:PlayerGraphics	= null;	// the graphics that we're controlling
		private var m_origX:Number				= 0.0;	// the original x position for the player
		private var m_origY:Number				= 0.0;	// the original y position for the player
		
		/**********************************************************************/
		
		/**
		 * The AIPlayer that this is for
		 */
		public function set aiPlayer( a:AIPlayer ):void
		{
			var len:int = this.m_anims.length;
			for( var i:int = 0; i < len; i++ )
				this.m_anims[i].aiPlayer = a;
		}
		
		/**********************************************************************/
		
		/**
		 * Creates a new AnimController
		 * @param graphics The graphics that we're controlling
		 */
		public function AnimController( graphics:PlayerGraphics ):void
		{
			this.m_graphics = graphics;
			
			// create our animations
			this.m_anims = new Vector.<Animation>();
			this.m_anims.push( new AnimationWin( Animations.WIN, graphics ) );
			this.m_anims.push( new AnimationLose( Animations.LOSE, graphics ) );
			this.m_anims.push( new AnimationCheckCards( Animations.CHECK_CARDS, graphics ) );
			this.m_anims.push( new AnimationCheckPot( Animations.CHECK_POT, graphics ) );
		}
		
		/**
		 * Destroys the AnimController and clears it for garbage collection
		 */
		public function destroy():void
		{
			// destroy our animations
			var len:int = this.m_anims.length;
			for( var i:int = 0; i < len; i++ )
				this.m_anims[i].destroy();
				
			// clear our properties
			this.m_anims 	= null;
			this.m_currAnim	= null;
			this.m_graphics	= null;
		}
		
		/**
		 * Sets the original position to use for the ai player
		 * @param x The original x position
		 * @param y The original y position
		 */
		public function setOriginalPos( x:Number, y:Number ):void
		{
			this.m_origX = x;
			this.m_origY = y;
		}
		
		/**
		 * Resets the player to it's original position and rotation etc
		 */
		public function resetPlayer():void
		{
			// stop any current animation
			if( this.m_currAnim != null )
			{
				this.m_currAnim.onFinish -= this._onAnimFinish;
				this.m_currAnim.stop();
				this.m_currAnim = null;
			}
			
			// reset our rotation and position
			this.m_graphics.rotation 	= 0.0;
			this.m_graphics.x			= this.m_origX;
			this.m_graphics.y			= this.m_origY;
		}
		
		/**
		 * Plays an animation, stopping any one that might be currently playing
		 * @param anim The anim to play; use the Animation constants
		 */
		public function playAnim( anim:String ):void
		{
			// find our anim
			var next:Animation 	= null;
			var len:int			= this.m_anims.length;
			for( var i:int = 0; i < len; i++ )
			{
				if( this.m_anims[i].name == anim )
				{
					next = this.m_anims[i];
					break;
				}
			}
			
			// if we can't find it
			if( next == null )
			{
				Console.print( "[AnimController] ERROR: Can't find the animation with the name '" + anim + "'" );
				return;
			}
			
			// reset the player
			this.resetPlayer();
			
			// start our new one
			this.m_currAnim 			= next;
			this.m_currAnim.onFinish	+= this._onAnimFinish;
			this.m_currAnim.play();
		}
		
		/**********************************************************************/
		
		// called when our current animation has finished
		private function _onAnimFinish():void
		{
			if( this.m_currAnim != null )
				this.m_currAnim.onFinish -= this._onAnimFinish;
			this.m_currAnim = null;
		}
	}
}