package logic.evaluators 
{
	import enums.CardValue;
	import enums.CardSuit;
	import enums.HandRank;
	import logic.Card;
	import logic.CardMatchResults;
	
	/**
	 * Evaluates a set of max 7 cards to find the best hand
	 */
	public class Evaluator
	{
		
		/**********************************************************************/
		
		private var m_evaluators:Vector.<BaseEvaluator> = null; // the list of our evaluators
		
		/**********************************************************************/
		
		/**
		 * Creates a new Evaluator. This is a singleton class, so use instance to access it
		 */
		public function Evaluator() 
		{
			// create the evaluators
			this.m_evaluators 							= new Vector.<BaseEvaluator>( HandRank.TOTAL );
			this.m_evaluators[HandRank.HIGH_CARD]		= new HighCardEvaluator( this );
			this.m_evaluators[HandRank.PAIR] 			= new PairEvaluator( this );
			this.m_evaluators[HandRank.TWO_PAIR] 		= new TwoPairEvaluator( this );
			this.m_evaluators[HandRank.TRIPS] 			= new TripsEvaluator( this );
			this.m_evaluators[HandRank.STRAIGHT] 		= new StraightEvaluator( this );
			this.m_evaluators[HandRank.FLUSH] 			= new FlushEvaluator( this );
			this.m_evaluators[HandRank.FULL_HOUSE] 		= new FullHouseEvaluator( this );
			this.m_evaluators[HandRank.POKER] 			= new PokerEvaluator( this );
			this.m_evaluators[HandRank.STRAIGHT_FLUSH]	= new StraightFlushEvaluator( this );
			this.m_evaluators[HandRank.ROYAL_FLUSH] 	= new RoyalFlushEvaluator( this );
		}
		
		/**
		 * Destroys the Evaluator and clears it for garbage collection
		 */
		public function destroy():void
		{
			// destroy our evaluators
			var len:int = this.m_evaluators.length;
			for( var i:int = 0; i < len; i++ )
				this.m_evaluators[i].destroy();
				
			// clear our properties
			this.m_evaluators = null;
		}
		
		/**
		 * Given a set of cards, it matches the best hand
		 * @param cards The cards that we're looking at
		 * @param matchResults The object to copy our results into
		 * @return The results of the match
		 */
		public function matchBestHand( cards:Vector.<Card>, matchResults:CardMatchResults ):CardMatchResults
		{			
			// go through our evaluators and return the first one that matches (highest, so reverse)
			for ( var i:int = this.m_evaluators.length - 1; i >= 0; i-- )
			{
				var results:CardMatchResults = this.m_evaluators[i].match( cards, matchResults );
				if ( results.success )
					return results;
			}
			return null; // shouldn't get here as we should at least have a high card match
		}
		
		/**
		 * Gets the highest card we have in our hand
		 * @param cards The cards that we're looking at
		 * @param matchResults The object to copy our match results into
		 * @return The results of the match
		 */
		public function matchHighCard( cards:Vector.<Card>, matchResults:CardMatchResults ):CardMatchResults
		{
			return this.m_evaluators[HandRank.HIGH_CARD].match( cards, matchResults );
		}
		
		/**
		 * Does a specific set of cards have a pair?
		 * @param cards The cards that we're looking at
		 * @param matchResults The object to copy our results into
		 * @param ignoreCard If we should ignore a specific card when trying to match this
		 * @return The results of the match
		 */
		public function matchPair( cards:Vector.<Card>, matchResults:CardMatchResults, ignoreCard:CardValue = null ):CardMatchResults
		{
			// set the ignore card and check for a match
			var pairEv:PairEvaluator 		= this.m_evaluators[HandRank.PAIR] as PairEvaluator;
			pairEv.ignoreCard 				= ignoreCard;
			var results:CardMatchResults	= pairEv.match( cards, matchResults );
			pairEv.ignoreCard				= null; // reset
			return results;
		}
		
		/**
		 * Does a specific set of cards have trips?
		 * @param cards The cards that we're looking at
		 * @param matchResults The object to copy our results into
		 * @return The results of the match
		 */
		public function matchTrips( cards:Vector.<Card>, matchResults:CardMatchResults ):CardMatchResults
		{
			return this.m_evaluators[HandRank.TRIPS].match( cards, matchResults );
		}
		
		/**
		 * Does a specific set of cards have 2 pair?
		 * @param cards The cards that we're looking at
		 * @param matchResults The object to copy our match results into
		 * @return The results of the match
		 */
		public function matchTwoPair( cards:Vector.<Card>, matchResults:CardMatchResults ):CardMatchResults
		{
			return this.m_evaluators[HandRank.TWO_PAIR].match( cards, matchResults );
		}
		
		/**
		 * Does a specific set of cards have a straight?
		 * @param cards The cards that we're looking at
		 * @param matchResults The object to copy our results into
		 * @param onlyCheckSuit Only check for a straight amongst a specific suit
		 * @return The results of the match
		 */
		public function matchStraight( cards:Vector.<Card>, matchResults:CardMatchResults, onlyCheckSuit:CardSuit = null ):CardMatchResults
		{
			var straightEv:StraightEvaluator 	= this.m_evaluators[HandRank.STRAIGHT] as StraightEvaluator;
			straightEv.onlyCheckSuit 			= onlyCheckSuit;
			var results:CardMatchResults		= straightEv.match( cards, matchResults );
			straightEv.onlyCheckSuit			= null; // reset
			return results;
		}
		
		/**
		 * Does a specific set of cards have a flush?
		 * @param cards The cards that we're looking at
		 * @param matchResults The object to copy our match results into
		 * @return The results of the match
		 */
		public function matchFlush( cards:Vector.<Card>, matchResults:CardMatchResults ):CardMatchResults
		{
			return this.m_evaluators[HandRank.FLUSH].match( cards, matchResults );
		}
		
		/**
		 * Does a specific set of cards have a full house?
		 * @param cards The cards that we're looking at
		 * @param matchResults The object to copy our match results into
		 * @return The results of the match
		 */
		public function matchFullHouse( cards:Vector.<Card>, matchResults:CardMatchResults ):CardMatchResults
		{
			return this.m_evaluators[HandRank.FULL_HOUSE].match( cards, matchResults );
		}
		
		/**
		 * Does a specific set of cards have a full house?
		 * @param cards The cards that we're looking at
		 * @param matchResults The object to copy our match results into
		 * @return The results of the match
		 */
		public function matchPoker( cards:Vector.<Card>, matchResults:CardMatchResults ):CardMatchResults
		{
			return this.m_evaluators[HandRank.POKER].match( cards, matchResults );
		}
		
		/**
		 * Does a specific set of cards have a straight flush?
		 * @param cards The cards that we're looking at
		 * @param matchResults The object to copy our match results into
		 * @return The results of the match
		 */
		public function matchStraightFlush( cards:Vector.<Card>, matchResults:CardMatchResults ):CardMatchResults
		{
			return this.m_evaluators[HandRank.STRAIGHT_FLUSH].match( cards, matchResults );
		}
		
		/**
		 * Does a specific set of cards have a royal flush?
		 * @param cards The cards that we're looking at
		 * @param matchResults The object to copy our match results into
		 * @return The results of the match
		 */
		public function matchRoyalFlush( cards:Vector.<Card>, matchResults:CardMatchResults ):CardMatchResults
		{
			return this.m_evaluators[HandRank.ROYAL_FLUSH].match( cards, matchResults );
		}
		
	}

}