package logic.evaluators 
{
	import enums.CardSuit;
	import enums.CardValue;
	import enums.HandRank;
	import logic.Card;
	import logic.CardMatchResults;
	
	/**
	 * Verifies if there's a straight in a given set of cards
	 */
	public class StraightEvaluator extends BaseEvaluator
	{
		
		/*******************************************************************************************/
		
		/**
		 * For when we only want to check against a specific suit
		 */
		public var onlyCheckSuit:CardSuit = null;
		
		/*******************************************************************************************/
		 
		/**
		 * Creates a new StraightEvaluator
		 * @param evaluator The main Evaluator class, that holds us
		 */
		public function StraightEvaluator( evaluator:Evaluator )
		{
			super( evaluator );
		}
		
		/**
		 * Takes a group of cards and sees if there's a match for a straight
		 * @param cards The cards that we're trying to match
		 * @param matchResults The object to copy our match results into
		 * @return The results of the match
		 */
		public function match( cards:Vector.<Card>, matchResults:CardMatchResults ):CardMatchResults
		{
			// reset our results object
			matchResults.reset();
			
			// quick check for card length
			var maxLen:int = 5;
			if ( cards.length < maxLen )
				return matchResults; // success should be false from the reset() call
				
			// sort the cards by value
			cards.sort( this._sort );
			
			// special case for ace
			var len:int 			= cards.length;
			var specialAce:Boolean	= false;
			if ( cards[0].value == CardValue.ACE )
			{
				cards.push( cards[0] ); // special case for ace, add it to the end as well
				len 		= cards.length;
				specialAce	= true;
			}
				
			var num:int = 1; // number of cards in a row that we have
			for ( var i:int = 1; i < len; i++ )
			{
				var card:Card 	= cards[i];		// this card
				var other:Card 	= cards[i - 1];	// the previous card
				
				// if we're only checking a specific suit
				if ( this.onlyCheckSuit != null && this.onlyCheckSuit != card.suit )
				{
					num = 1;
					matchResults.bestHand.clear();
					continue;
				}
				
				// check the cards
				if ( ( other.value == ( card.value + 1 ) ) || // prev card was 1 plus this one
					( card.value == CardValue.ACE && other.value == CardValue.TWO ) ) // special case for ace
				{
					matchResults.bestHand.push( other );
					num++;
					if ( num == maxLen )
					{
						matchResults.bestHand.push( card );
						
						// it's a match, set up the match results, and store them
						matchResults.type		= HandRank.STRAIGHT;
						matchResults.success	= true;
						
						// if this was the special ace case, remove the last one
						if ( specialAce )
							cards.pop();
					
						// return our results
						return matchResults;
					}
				}
				else if ( other.value != card.value ) // could have 6 cards, two the same
				{
					num = 1;
					matchResults.bestHand.clear();
				}
			}
						
			// if this was the special ace case, remove the last one
			if ( specialAce )
				cards.pop();
			
			// no match
			return matchResults; // success should be false from the reset() call
		}
		
		/*******************************************************************************************/
		
		/**
		 * Sorts 2 cards based on their index
		 * @param a The first card
		 * @param b The second card
		 * @return If the first card is higher than the second
		 */
		private function _sort( a:Card, b:Card ):int
		{
			return b.value - a.value;
		}
		
	}

}