package state
{
	import Loom.GameFramework.ITicked;
	import Loom.GameFramework.TimeManager;
	import Loom.Utils.Injector;
	
	public delegate GameStateManagerCallback( oldState:GameState, newState:GameState ):void;

	/**
	 * A GameStateManager class - controls the game states that we have in the game
	 */
	public class GameStateManager implements ITicked
	{
		
		/**********************************************************************/
		
		/**
		 * Called when the state has changed. It should take two paramters of type GameState:
		 * the old state (which can be null) and the new state
		 */
		public var onStateChanged:GameStateManagerCallback = null;
		
		/**
		 * The time manager for the game
		 */
		[Inject]
		public var timeManager:TimeManager = null;
		
		/**********************************************************************/
		
		private var m_states:Vector.<GameState> = null; // the states that we're managing
		private var m_currState:GameState		= null;	// our current state	
		private var m_nextState:GameState		= null;	// the next state that we're moving to
		private var m_injector:Injector			= null;	// the injector for our states
		private var m_addedToUpdate:Boolean		= false;// have we been added to the update yet?
		private var m_isChanging:Boolean		= false;// are we changing yet?
		private var m_param:Object				= null;	// the param to pass to the state
		
		/**********************************************************************/
		
		/**
		 * The current state that we're in
		 */
		public function get currState():GameState { return this.m_currState; }
		
		/**
		 * The current state's name, or null if we don't have a current state
		 */
		public function get currStateName():String { return ( this.m_currState != null ) ? this.m_currState.name : null; }
		
		/**********************************************************************/
		
		/**
		 * Creates a new GameStateManager
		 * @param injector The Injector that we're using for filling our GameState params
		 */
		public function GameStateManager( injector:Injector )
		{
			this.m_states 	= new Vector.<GameState>();
			this.m_injector	= injector;
		}
		
		/**
		 * Calls reset() on all our states
		 */
		public function reset():void
		{
			var len:int = this.m_states.length;
			for( var i:int = 0; i < len; i++ )
				this.m_states[i].reset();
		}
		
		/**
		 * Destroys the GameStateManager and clears it for garbage collection
		 */
		public function destroy():void
		{
			// remove us from the time manager
			this.timeManager.removeTickedObject( this );
			this.m_addedToUpdate = false;
			
			// destroy all our game states
			var len:int = this.m_states.length;
			for( var i:int = 0; i < len; i++ )
				this.m_states[i].destroy();
				
			// clear our properties
			this.onStateChanged	= null;
			this.timeManager	= null;
			this.m_states 		= null;
			this.m_currState	= null;
			this.m_nextState	= null;
			this.m_injector		= null;
		}
		
		/**
		 * All states have the injector applied when they're added. This lets us re-inject
		 * them if needed
		 */
		public function inject():void
		{
			var len:int = this.m_states.length;
			for( var i:int = 0; i < len; i++ )
				this.m_injector.apply( this.m_states[i] );
		}
		
		/**
		 * Adds a GameState to be managed
		 * @param state The GameState that we want to be managed
		 */
		public function addState( state:GameState ):void
		{
			// make sure it's not already there
			if( this.m_states.indexOf( state ) != -1 )
			{
				Console.print( "[GameStateManager] ERROR: Can't add state '" + state.name + "', as we're already managing it" );
				return;
			}
			
			// add it
			this.m_injector.apply( state );
			this.m_states.push( state );
			state.init();
		}
		
		/**
		 * Changes the state
		 * @param name The name of the state that we want to change to
		 * @param param An optional param to pass to the state for it's begin() call
		 */
		public function changeState( name:String, param:Object = null ):void
		{
			// add us to the time manager if we're not already there
			if( !this.m_addedToUpdate )
			{
				this.timeManager.addTickedObject( this );
				this.m_addedToUpdate = true;
			}
			
			// find our state
			var len:int 			= this.m_states.length;
			var newState:GameState	= null;
			for( var i:int = 0; i < len; i++ )
			{
				if( this.m_states[i].name == name )
				{
					newState = this.m_states[i];
					break;
				}
			}
			
			// make sure we have it
			if( newState == null )
			{
				Console.print( "[GameStateManager] ERROR: Can't change to state '" + name + "' as we don't have a GameState with that name" );
				return;
			}
			
			// if it's the same state, do nothing
			if( newState == this.m_currState )
			{
				Console.print( "[GameStateManager] ERROR: Not changing to state '" + name + "' as it's already the current state" );
				return;
			}
			
			// set our next state - it'll change at the end of the update
			this.m_nextState 	= newState;
			this.m_param		= param;
			this.m_isChanging	= false;
		}
		
		/**
		 * Called when we're updating the GameStateManager
		 */
		public function onTick():void
		{
			// if we have a current state, update it
			if( this.m_currState != null )
				this.m_currState.onTick();
				
			// if we're changing state...
			if ( this.m_nextState == null || this.m_isChanging )
				return;
			
			// add our listener to onEnd
			if( this.m_currState == null )
				this._changeToNextState(); // just change
			else
			{
				this.m_isChanging 		= true; // set that we're changing so we don't call this multiple times
				this.m_currState.onEnd	+= this._changeToNextState; // add our listener for when it's finished
				this.m_currState.end();
			}
		}
		
		/**********************************************************************/
		
		// changes to the next state
		private function _changeToNextState():void
		{
			// make sure we have one
			if( this.m_nextState == null )
				return;
			
			// hold our old state (end should have already been called)
			var oldState:GameState = this.m_currState;
			if( oldState != null )
				oldState.onEnd -= this._changeToNextState; // remove our listener
				
			// set our new current state
			var prevName:String	= ( oldState != null ) ? oldState.name : "null";
			var param:Object	= this.m_param; // in case we change states in the begin() call, in which case, we'd null m_param
			this.m_currState 	= this.m_nextState;
			this.m_nextState	= null;
			this.m_param		= null;
			Console.print( "[GameStateManager] Starting game state '" + this.m_currState.name + "', prev state: '" + prevName + "'" );
			this.m_currState.begin( param );
			
			// dispatch our signal
			this.onStateChanged( oldState, this.m_currState );
		}
		
	}
	
}