package state
{
	import constants.PokerStates;
	import logic.BetLogic;
	import logic.Card;
	import logic.Player;

	/**
	 * The betting state, where the two players can check, raise or fold
	 */
	public class PokerStateBet extends PokerGameState
	{
		
		/**********************************************************************/
		
		private var m_betLogic:BetLogic			= null;	// the bet logic object
		private var m_currTime:Number			= 0.0;	// our current timer
		private var m_waitTime:Number			= 1.0;	// how long to wait for
		private var m_nextState:String			= null;	// the next state to go to - use the poker states consts
		private var m_betting:Boolean			= false;// are we currently betting?
		private var m_playerCards:Vector.<Card>	= null;	// the vector of cards that a player can use (hole + table cards)
		
		/**********************************************************************/
		
		/**
		 * Creates a new PokerStateBet
		 */
		public function PokerStateBet()
		{
			super( PokerStates.BET );
			
			// create our bet logic object
			this.m_betLogic 	= new BetLogic( this );
			this.m_playerCards	= new Vector.<Card>();
		}
		
		/**
		 * Destroys the PokerStateBet and clears it for garbage collection
		 */
		override public function destroy():void
		{
			super.destroy();
			
			// destroy our logic
			this.m_betLogic.destroy();
			
			// clear our properties
			this.m_betLogic		= null;
			this.m_playerCards	= null;
		}
		
		/**
		 * Called when we're coming into this state - choose our dealer
		 * @param param An optional param
		 */
		override public function begin( param:Object = null ):void
		{
			super.begin( param );
			
			// check that our param is a good state
			var currState:String = ( param as String );
			if( !PokerStates.isValid( currState ) )
			{
				Console.print( "[PokerStateBet] ERROR: The param passed (" + param + ") isn't a valid PokerState name" );
				return;
			}
			
			// set our next state
			this.m_nextState = PokerStates.getNextState( currState );
			if( this.m_nextState == null )
			{
				Console.print( "[PokerStateBet] ERROR: Couldn't find the next state" );
				return;
			}
			
			// calculate both player's hands
			this._fillPlayerCards( this.mainPlayer );
			this.cardEvaluator.matchBestHand( this.m_playerCards, this.mainPlayer.cardMatchResults );
			this._fillPlayerCards( this.opponent )
			this.cardEvaluator.matchBestHand( this.m_playerCards, this.opponent.cardMatchResults );
			
			// reset our current time
			this.m_currTime	= 0.0;
			
			// if either player has no cash (all-in), then move on
			if( this.mainPlayer.money == 0 || this.opponent.money == 0 )
				this.m_betting = false;
			else
			{
				this.m_betting	= true;
				
				// tell our bet logic to begin
				this.m_betLogic.onFinish += this._onFinish;
				this.m_betLogic.begin( PokerStates.getBetRound( currState ) );
			}
		}
		
		/**
		 * Called when we're ending this state - clean up
		 */
		override public function end():void
		{
			super.end();
			
			// tell our bet logic to end
			this.m_betLogic.onFinish -= this._onFinish;
			this.m_betLogic.end();
			
			// if either of our players are all-in, then show the opponent's cards
			if( ( this.mainPlayer.money == 0 || this.opponent.money == 0 ) && this.m_betLogic.foldingPlayer == null )
				this.opponent.showHoleCards();
		}
		
		/**
		 * Called every tick we're active - wait a bit before changing
		 */
		override public function onTick():void
		{
			super.onTick();
			
			// if we're betting, do nothing
			if( this.m_betting )
				return;
				
			// wait for a bit
			var dt:Number	= this.timeManager.TICK_RATE;
			this.m_currTime	+= dt;
			if( this.m_currTime < this.m_waitTime )
				return;
				
			// if one of the players folded, just move to the winner state, otherwise keep going
			if( this.m_betLogic.foldingPlayer != null )
				this.stateManager.changeState( PokerStates.WINNER, this.m_betLogic.foldingPlayer );
			else
				this.stateManager.changeState( this.m_nextState );
		}
		
		/**********************************************************************/
		
		// creates the cards that a player can use
		private function _fillPlayerCards( player:Player ):void
		{
			this.m_playerCards.clear();
			
			// add the table cards
			var len:int = this.deck.tableCards.length;
			for( var i:int = 0; i < len; i++ )
				this.m_playerCards.push( this.deck.tableCards[i] );
				
			// add the player's hole cards
			this.m_playerCards.push( player.holeCard1 );
			this.m_playerCards.push( player.holeCard2 );
		}
		
		// called when the bet logic is finished
		private function _onFinish():void
		{
			this.m_betting 	= false;
			this.m_currTime	= 0.0;
		}
		
	}
	
}