package state
{
	import constants.Animations;
	import constants.PokerStates;
	import enums.ResultType;
	import logic.Card;
	import logic.Player;

	/**
	 * The winner state, where we decide who won the pot
	 */
	public class PokerStateWinner extends PokerGameState
	{
		
		/**********************************************************************/
		
		private var m_currTime:Number			= 0.0;	// our current timer
		private var m_waitTime:Number			= 0.0;	// how long to wait for
		private var m_winner:Player				= null;	// the winning player
		private var m_isShowingCards:Boolean	= false;// are we showing our cards?
		
		/**********************************************************************/
		
		/**
		 * Creates a new PokerStateWinner
		 */
		public function PokerStateWinner()
		{
			super( PokerStates.WINNER );
		}
		
		/**
		 * Called when we're coming into this state - choose our dealer
		 * @param param An optional param
		 */
		override public function begin( param:Object = null ):void
		{
			super.begin( param );
			
			// check do we have a folding player
			var foldingPlayer:Player = ( param as Player );
			
			// if there's no folding player, then we need to show the ai player's cards
			// and decide who has the better hand
			var aiReveal:Boolean = false;
			if( foldingPlayer == null )
			{
				// the ai has to show the cards
				aiReveal = true;
				
				// figure out the winner (cardMatchResults will have been updated in the last bet stage)
				var resultType:ResultType = this.mainPlayer.cardMatchResults.compareAgainst( this.opponent.cardMatchResults );
				if( resultType == ResultType.WIN )
					this.m_winner = this.mainPlayer;
				else if( resultType == ResultType.LOSE )
					this.m_winner = this.opponent;
				else
					this.m_winner = null; // split pot
				
				// show the winning hand
				var bestHand:Vector.<Card> = this.mainPlayer.cardMatchResults.bestHand;
				if( this.m_winner == this.opponent )
					bestHand = this.opponent.cardMatchResults.bestHand;
				this.deck.showWinningHand( bestHand );
			}
			else
			{
				// if the folding player is the main player, check if the ai should reveal their cards
				if( foldingPlayer == this.mainPlayer )
				{
					this.m_winner	= this.opponent;
					aiReveal 		= this.opponent.ai.shouldRevealHoleCards();
				}
				else
					this.m_winner = this.mainPlayer;
			}
			
			// if we should reveal the ai's cards, do it
			if( aiReveal )
				this.opponent.showHoleCards();
				
			// tell which player that they won the hand
			if( this.m_winner == null )
			{
				this.mainPlayer.winHand( false, true ); // did opponent fold, is split pot
				this.opponent.winHand( false, true ); 	// did player fold, is split pot
			}
			else
				this.m_winner.winHand( ( foldingPlayer != null ), false ); // did losing player fold, is split pot
			
			// reset our current time
			this.m_isShowingCards	= aiReveal;
			this.m_currTime			= 0.0;
			this.m_waitTime			= ( foldingPlayer == null ) ? 5.0 : 2.0;
		}
		
		/**
		 * Called every tick we're active - wait a bit before changing
		 */
		override public function onTick():void
		{
			super.onTick();
			
			// wait for a bit
			var dt:Number	= this.timeManager.TICK_RATE;
			this.m_currTime	+= dt;
			if( this.m_currTime < this.m_waitTime )
				return;
				
			// if the winner is null, then it's a split pot
			if( this.m_winner == null )
			{
				// split pot - if it's uneven, favour the player
				var oppHalf:int 	= this.mainPot.currAmount / 2;
				var playerHalf:int	= ( ( oppHalf + oppHalf != this.mainPot.currAmount ) ? oppHalf + 1 : oppHalf );
				this.mainPlayer.winCash( playerHalf );
				this.opponent.winCash( oppHalf );
				Console.print( "[PokerStateWinner] Opponent split the pot and won " + oppHalf + ", money: " + this.opponent.money );
				Console.print( "[PokerStateWinner] Player split the pot and won " + playerHalf + ", money: " + this.mainPlayer.money );
			}
			else
			{
				// play an animation on the opponent - but only if they had to show their cards
				if( this.m_isShowingCards )
				{
					if( this.m_winner == this.opponent )
						this.opponent.playAnim( Animations.WIN );
					else if( this.m_winner == this.mainPlayer )
						this.opponent.playAnim( Animations.LOSE );
				}
				
				var amount:int = this.mainPot.currAmount;
				this.m_winner.winCash( amount ); // the winner gets the pot
				Console.print( "[PokerStateWinner] " + this.m_winner + " won " + amount + ", money: " + this.m_winner.money );
			}
			
			// clean up
			this.m_isShowingCards	= false;
			this.m_currTime			= 0.0;
			this.m_waitTime			= 1.0;
			this.deck.gatherCards();
			
			// if either the player or the opponent has no money left, then it's game over
			if( this.opponent.money <= 0 )
			{
				Console.print( "[PokerStateWinner] The player has won!" );
				this.stateManager.changeState( PokerStates.END_MATCH, true );
			}
			else if( this.mainPlayer.money <= 0 )
			{
				Console.print( "[PokerStateWinner] The player has lost!" );
				this.stateManager.changeState( PokerStates.END_MATCH, false );
			}
			else
				this.stateManager.changeState( PokerStates.DEALER ); // move onto the next round
		}
		
	}
	
}