package state
{
	import cocos2d.ccColor3B;
	
	import Loom.Animation.Tween;
	import Loom.Animation.EaseType;

	import constants.Resources;
	import constants.States;
	import gui.Btn;
	import gui.Image;
	import gui.PlayerBtn;
	import gui.TextField;
	import gui.popups.PopupCharity;
	import logic.AIPlayer;
	import logic.PlayerManager;

	/**
	 * The main menu state, where we see the different buttons etc to choose our opponent
	 */
	public class StateMainMenu extends MainGameState
	{
		
		/**********************************************************************/
		
		/**
		 * The PlayerManager that controls all the players
		 */
		[Inject]
		public var playerManager:PlayerManager = null;
		
		/**********************************************************************/
		
		private var m_bg:Image					= null;	// the image for the bg
		private var m_title:Image		 		= null; // the image logo for the game
		private var m_txtInstructions:TextField	= null; // the instructions for this menu
		private var m_txtMoney:TextField		= null; // the money of the player
		private var m_btns:Vector.<PlayerBtn>	= null; // the buttons for the players
		private var m_popup:PopupCharity		= null;	// the popup for when we're giving the player money because they have none
		
		/**********************************************************************/
		
		/**
		 * Creates a new StateMainMenu
		 */
		public function StateMainMenu()
		{
			super( States.MENU );
			
			// define our text colour
			var colour:ccColor3B = new ccColor3B( 33, 29, 208 );
			
			// create our objects
			this.m_bg 				= new Image( Resources.ATLAS_BACKGROUNDS, Resources.SPRITE_MAIN_MENU_BG );
			this.m_title	 		= new Image( Resources.ATLAS_SPRITES, Resources.SPRITE_GAME_LOGO );
			this.m_txtInstructions	= new TextField( "Choose your opponent", Resources.FONT_CURSE, colour );
			this.m_txtMoney			= new TextField( "Cash money: 0", Resources.FONT_CURSE, colour );
			
			// set our text scale
			this.m_txtInstructions.scale 	= 0.25;
			this.m_txtMoney.scale 			= 0.4;
		}
		
		/**
		 * Inits the main menu state - at this point, any injection is complete
		 */
		override public function init():void
		{
			// create our player buttons
			this.m_btns	= new Vector.<PlayerBtn>();
			var len:int = this.playerManager.numPlayers;
			for( var i:int = 0; i < len; i++ )
			{
				var aiPlayer:AIPlayer	= this.playerManager.getPlayerByIndex( i );
				var btn:PlayerBtn 		= new PlayerBtn( aiPlayer );
				this.m_btns.push( btn );
			}
			
			// create our popup
			var minAmount:int 	= this.playerManager.getPlayerByIndex( 0 ).buyIn;
			this.m_popup 		= new PopupCharity( this.guiCanvas.designWidth, this.guiCanvas.designHeight, minAmount );
		}
		
		/**
		 * Destroys the StateMainMenu and clears it for garbage collection
		 */
		override public function destroy():void
		{
			super.destroy();
			
			// destroy any objects
			this.m_bg.destroy();
			this.m_title.destroy();
			this.m_txtInstructions.destroy();
			this.m_txtMoney.destroy();
			this.m_popup.destroy();
			
			// destroy our buttons
			var len:int = this.m_btns.length;
			for( var i:int = 0; i < len; i++ )
				this.m_btns[i].destroy();
			
			// null our properties
			this.m_bg				= null;
			this.m_title	 		= null;
			this.m_txtInstructions	= null;
			this.m_txtMoney			= null;
			this.m_btns				= null;
			this.m_popup			= null;
		}
		
		/**
		 * Called when we're coming into this state - show our objects
		 * @param param An optional param
		 */
		override public function begin( param:Object = null ):void
		{
			super.begin( param );
			
			// check if we need to give the main player money
			var minAmount:int 			= this.playerManager.getPlayerByIndex( 0 ).buyIn;
			var didGiveCharity:Boolean	= false;
			if( this.mainPlayer.totalMoney < minAmount )
			{
				this.mainPlayer.totalMoney = minAmount;
				if( this.mainPlayer.statsNumHands != 0 )
				{
					Console.print( "[StateMainMenu] Giving the main player some cash as they don't have enough to play" );
					this.mainPlayer.statsNumCharity += 1;
					didGiveCharity = true;
				}
			}
			
			// position our bg
			this.m_bg.y = this.guiCanvas.designHeight;
			
			// position our title
			var desiredX:Number	= this.guiCanvas.designWidth * 0.5 -this.m_title.width * 0.5;
			this.m_title.x 		= -this.m_title.width;
			this.m_title.y 		= this.guiCanvas.designHeight - 2.0;
			
			// position our instructions
			this.m_txtInstructions.x 		= this.guiCanvas.designWidth * 0.5 - this.m_txtInstructions.width * 0.5;
			this.m_txtInstructions.y 		= this.m_title.y - this.m_title.height + this.m_txtInstructions.height * 2.5;
			this.m_txtInstructions.alpha	= 0;
			
			// position our money
			this.m_txtMoney.text	= "Cash money: " + this.mainPlayer.totalMoney;
			this.m_txtMoney.x		= this.guiCanvas.designWidth * 0.5 - this.m_txtMoney.width * 0.5;
			this.m_txtMoney.y		= this.m_txtMoney.height * 2.0;
			this.m_txtMoney.alpha	= 0;
			
			// tween our objects
			var time:Number = 1.0;
			Tween.to( this.m_title, time, { "x":desiredX, "ease":EaseType.EASE_OUT_ELASTIC } );
			Tween.to( this.m_txtInstructions, time, { "alpha":1.0 } );
			Tween.to( this.m_txtMoney, time, { "alpha":1.0 } );
			
			// position our buttons
			var len:int 	= this.m_btns.length;
			var i:int		= 0;
			var tw:Number	= 0.0;
			for( i = 0; i < len; i++ )
				tw += this.m_btns[i].width; // get the total width first
				
			// get our spacing between the buttons
			var spacing:int = int( ( this.guiCanvas.designWidth - tw ) / ( len + 1 ) );
			
			// position, and tween our buttons
			var x:int = spacing;
			var y:int = this.m_txtInstructions.y - this.m_txtInstructions.height * 3.5;
			for( i = 0; i < len; i++ )
			{
				this.m_btns[i].x 			= x;
				this.m_btns[i].y 			= 0.0;
				this.m_btns[i].enabled 		= ( this.m_btns[i].buyIn <= this.mainPlayer.totalMoney );
				this.m_btns[i].onBtnClick	+= this._onBtnClick;
				
				Tween.to( this.m_btns[i], time, { "y":y, "ease":EaseType.EASE_OUT_BACK, "delay":0.1 * i } );
				
				// move our x position along
				x += this.m_btns[i].width + spacing;
			}
			
			// add our other elements
			this.guiCanvas.addChild( this.m_bg );
			this.guiCanvas.addChild( this.m_title );
			this.guiCanvas.addChild( this.m_txtInstructions );
			this.guiCanvas.addChild( this.m_txtMoney );
			for( i = 0; i < len; i++ )
				this.guiCanvas.addChild( this.m_btns[i] );
			
			// if we gave charity, show the player
			if( didGiveCharity )
			{
				this.m_popup.show();
				this.guiCanvas.addChild( this.m_popup );
			}
		}
		
		/**
		 * Called when we're leaving this state - remove our objects
		 */
		override public function end():void
		{
			// super.end(); // don't call super here, as we're going to dispatch onEnd() when the tweens have finished
			
			// make sure our popup isn't showing
			this.m_popup.hide();
			
			var len:int = this.m_btns.length;
			var i:int	= 0;
			
			// stop any tweens
			Tween.killTweensOf( this.m_title );
			Tween.killTweensOf( this.m_txtInstructions );
			Tween.killTweensOf( this.m_txtMoney );
			for( i = 0; i < len; i++ )
			{
				Tween.killTweensOf( this.m_btns[i] );
				this.m_btns[i].onBtnClick -= this._onBtnClick;
			}
			
			// tween out
			var time:Number = 0.5;
			Tween.to( this.m_title, time, { "x":-this.m_title.width, "ease":EaseType.EASE_IN_BACK } );
			Tween.to( this.m_txtInstructions, time, { "alpha":0.0 } );
			Tween.to( this.m_txtMoney, time, { "alpha":0.0 } );
			for( i = 0; i < len; i++ )
			{
				var t:Tween = Tween.to( this.m_btns[i], time, { "y":0.0, "ease":EaseType.EASE_IN_BACK, "delay":0.1 * i } );
				if( i == len - 1 )
					t.onComplete += this._onTweensFinished;
			}
		}
		
		/**********************************************************************/
		
		// called when a button has been clicked
		private function _onBtnClick( btn:Btn ):void
		{
			// get our button
			var playerBtn:PlayerBtn = ( btn as PlayerBtn );
			if( playerBtn == null )
			{
				Console.print( "[StateMainMenu] ERROR: Couldn't find player button after clicking on button " + btn );
				return;
			}
			
			// switch states
			Console.print( "[StateMainMenu] Choosen player " + playerBtn.aiPlayer );
			this.stateManager.changeState( States.PLAY, playerBtn.aiPlayer );
		}
		
		// called when all the tweens have finished and we can change states
		private function _onTweensFinished( t:Tween ):void
		{
			// remove our elements
			this.guiCanvas.removeChild( this.m_bg, false );
			this.guiCanvas.removeChild( this.m_title, false );
			this.guiCanvas.removeChild( this.m_txtInstructions, false );
			this.guiCanvas.removeChild( this.m_txtMoney, false );
			
			// remove our buttons
			var len:int = this.m_btns.length;
			for( var i:int = 0; i < len; i++ )
				this.guiCanvas.removeChild( this.m_btns[i] );
				
			// finish
			this.onEnd();
		}
		
	}
	
}