package state
{
	import cocos2d.CCSprite;
	
	import Loom.Animation.Tween;
	import Loom.Animation.EaseType;
	import Loom.Utils.Injector;

	import constants.PokerStates;
	import constants.States;
	import constants.Resources;
	import gui.DealerButton;
	import gui.Image;
	import gui.PlayerGraphics;
	import gui.MsgBar;
	import gui.MoneyStack;
	import gui.PlayActionButtons;
	import gui.Table;
	import logic.AI;
	import logic.AIPlayer;
	import logic.CardMatchResults;
	import logic.Deck;
	import logic.Player;
	import logic.evaluators.Evaluator;

	/**
	 * The play state, where we play against our opponent
	 */
	public class StatePlay extends MainGameState
	{
		
		/**********************************************************************/
		
		private var m_bg:Image								= null;	// the bg sprite
		private var m_opponent:AIPlayer						= null; // the player that we're playing
		private var m_opponentGraphics:PlayerGraphics		= null;	// the graphics for our opponent
		private var m_opponentAI:AI							= null;	// the ai for our opponent
		private var m_deck:Deck						 		= null;	// the deck that we'll use in the game
		private var m_table:Table							= null; // the table for the game
		private var m_potPlayer:MoneyStack					= null;	// the money stack for the main player
		private var m_potOpponent:MoneyStack				= null;	// the money stack for the opponent
		private var m_pot:MoneyStack						= null;	// the money stack for the pot
		private var m_buttons:PlayActionButtons				= null; // the action buttons
		private var m_dealerButton:DealerButton				= null;	// the dealer button to show who's the dealer
		private var m_msgBar:MsgBar							= null; // the message bar, to show messages to the player
		private var m_injector:Injector						= null;	// the injector, to inject our values into the poker states
		private var m_pokerStateManager:GameStateManager	= null;	// the poker state manager
		private var m_cardEvaluator:Evaluator				= null;	// the card evaluator to find our best hand
		
		/**********************************************************************/
		
		/**
		 * Creates a new StatePlay
		 */
		public function StatePlay()
		{
			super( States.PLAY );
			
			// create our objects
			this.m_bg 				= new Image( Resources.ATLAS_BACKGROUNDS, Resources.SPRITE_PLAY_BG );
			this.m_opponentGraphics	= new PlayerGraphics();
			this.m_deck 			= new Deck();
			this.m_table 			= new Table( this.m_deck );
			this.m_potPlayer		= new MoneyStack( 100, 100 );
			this.m_potOpponent		= new MoneyStack( 100, 100 );
			this.m_pot				= new MoneyStack( 0, 100 );
			this.m_buttons			= new PlayActionButtons;
			this.m_cardEvaluator	= new Evaluator;
			
			// position our money stacks
			this.m_potPlayer.x		= 190.0;
			this.m_potPlayer.y		= 10.0;
			this.m_potOpponent.x	= 260.0;
			this.m_potOpponent.y	= 140.0;
			this.m_pot.x			= 100.0;
			this.m_pot.y			= 75.0;
		}
		
		/**
		 * Inits the state play - this is called after the injection is done
		 */
		override public function init():void
		{
			super.init();
			
			// create our opponent ai
			this.m_opponentAI = new AI( this.timeManager );
			
			// tell our play action buttons about the canvas
			this.m_buttons.guiCanvas = this.guiCanvas;
			
			// create our graphics that need the canvas height
			this.m_dealerButton = new DealerButton( this.guiCanvas.designHeight );
			this.m_msgBar		= new MsgBar( this.guiCanvas.designHeight );
			
			// create our injector and map our values
			this.m_injector = new Injector;
			this.m_injector.mapValue( this.timeManager );
			this.m_injector.mapValue( this.mainPlayer );
			this.m_injector.mapValue( this.guiCanvas );
			this.m_injector.mapValue( this.m_deck );
			this.m_injector.mapValue( this.m_buttons );
			this.m_injector.mapValue( this.m_pot );
			this.m_injector.mapValue( this.m_cardEvaluator );
			
			// create our state manager
			this.m_pokerStateManager 				= new GameStateManager( this.m_injector );
			this.m_pokerStateManager.timeManager	= this.timeManager;
			this.m_injector.mapValue( this.m_pokerStateManager );
			
			// create our states
			this.m_pokerStateManager.addState( new PokerStateBet );
			this.m_pokerStateManager.addState( new PokerStateDealer );
			this.m_pokerStateManager.addState( new PokerStatePreFlop );
			this.m_pokerStateManager.addState( new PokerStateFlop );
			this.m_pokerStateManager.addState( new PokerStateTurn );
			this.m_pokerStateManager.addState( new PokerStateRiver );
			this.m_pokerStateManager.addState( new PokerStateWinner );
			this.m_pokerStateManager.addState( new PokerStateEndMatch );
			this.m_pokerStateManager.addState( new PokerStateWait );
		}
		
		/**
		 * Destroys the StatePlay and clears it for garbage collection
		 */
		override public function destroy():void
		{
			super.destroy();
			
			// destroy any objects
			this.m_opponentGraphics.destroy();
			this.m_opponentAI.destroy();
			this.m_deck.destroy();
			this.m_table.destroy();
			this.m_potPlayer.destroy();
			this.m_potOpponent.destroy();
			this.m_pot.destroy();
			this.m_buttons.destroy();
			this.m_pokerStateManager.destroy();
			this.m_dealerButton.destroy();
			this.m_msgBar.destroy();
			this.m_cardEvaluator.destroy();
			
			// null our properties
			this.m_bg					= null;
			this.m_opponent				= null;
			this.m_opponentGraphics		= null;
			this.m_opponentAI			= null;
			this.m_deck					= null;
			this.m_table				= null;
			this.m_potPlayer			= null;
			this.m_potOpponent			= null;
			this.m_pot					= null;
			this.m_buttons				= null;
			this.m_injector				= null;
			this.m_pokerStateManager	= null;
			this.m_dealerButton			= null;
			this.m_msgBar				= null;
			this.m_cardEvaluator		= null;
		}
		
		/**
		 * Called when we're coming into this state - show our objects
		 * @param param An optional param - should be an AIPlayer object
		 */
		override public function begin( param:Object = null ):void
		{
			super.begin( param );
			
			// make sure we have an AIPlayer
			this.m_opponent = ( param as AIPlayer );
			if( this.m_opponent == null )
			{
				Console.print( "[StatePlay] ERROR: Can't start state as the param passed (" + param + ") isn't an AIPlayer object" );
				return;
			}
			
			// map the opponent and re-inject our poker states so they're updated
			this.m_injector.mapValue( this.m_opponent );
			this.m_pokerStateManager.inject();
			this.m_pokerStateManager.reset();
			
			// set our opponent ai
			this.m_opponentAI.mainPlayer	= this.mainPlayer;
			this.m_opponentAI.aiPlayer 		= this.m_opponent;
			this.m_opponent.ai				= this.m_opponentAI;
			
			// set our opponent's money, and remove it from the player
			this.m_opponent.money 	= this.m_opponent.buyIn;
			this.mainPlayer.money	= this.m_opponent.money;
			
			// add our cash callbacks to our players
			this.mainPlayer.onIsDealer	+= this._onPlayerIsDealer;
			this.mainPlayer.onPayCash 	+= this._onMainPlayerPayCash;
			this.mainPlayer.onWinCash 	+= this._onMainPlayerWinCash;
			this.mainPlayer.onWinHand 	+= this._onMainPlayerWinHand;
			this.m_opponent.onIsDealer	+= this._onPlayerIsDealer;
			this.m_opponent.onPayCash 	+= this._onOpponentPayCash;
			this.m_opponent.onWinCash 	+= this._onOpponentWinCash;
			this.m_opponent.onWinHand 	+= this._onOpponentWinHand;
			
			// position our bg
			this.m_bg.y = this.guiCanvas.designHeight;
			
			// reset our dealer button
			this.m_dealerButton.reset();
			
			// set our player graphics and position them
			this.m_opponentGraphics.aiPlayer 	= this.m_opponent;
			this.m_opponentGraphics.x			= this.guiCanvas.designWidth;
			this.m_opponentGraphics.y			= 160.0;
			
			// position our table
			this.m_table.x	= this.guiCanvas.designWidth - this.m_table.width;
			this.m_table.y	= 0.0;
			this.m_table.reset();
			
			// set our pots
			this.m_potPlayer.maxAmount 		= this.mainPlayer.money + this.m_opponent.money;
			this.m_potOpponent.maxAmount	= this.m_potPlayer.maxAmount;
			this.m_pot.maxAmount			= this.m_potPlayer.maxAmount;
			this.m_potPlayer.reset();
			this.m_potOpponent.reset();
			this.m_pot.reset();
			
			// set the big blind on our action buttons
			this.m_buttons.bigBlind = this.m_opponent.bigBlind;
			this.m_buttons.maxBet	= this.mainPlayer.money;
			
			// add our elements
			this.guiCanvas.addChild( this.m_bg );
			this.guiCanvas.addChild( this.m_opponentGraphics );
			this.guiCanvas.addChild( this.m_table );
			this.guiCanvas.addChild( this.m_potOpponent );
			this.guiCanvas.addChild( this.m_pot );
			this.guiCanvas.addChild( this.m_dealerButton );
			this.guiCanvas.addChild( this.m_potPlayer );
			this.guiCanvas.addChild( this.m_msgBar );
			
			// add our poker state manager delegate listener
			this.m_pokerStateManager.onStateChanged += this._onPokerStateChange;
			
			// tween our objects
			Tween.to( this.m_table, 1.0, { "y":this.m_table.height, "ease":EaseType.EASE_OUT } ).onComplete += this._onBeginTweensFinished;
		}
		
		/**
		 * Called when we're leaving this state - remove our objects
		 */
		override public function end():void
		{
			// super.end(); // don't call super here, as we're going to dispatch onEnd() when the tweens have finished
			
			// clear the ai
			this.m_opponentAI.aiPlayer	= null;
			this.m_opponent.ai			= null;
			
			// remove the buyin, then add whatever the player won (maybe nothing.
			// NOTE: we remove the buy-in here, as we don't have persistence for mid-matches, only
			// the current level etc - so if we removed it at the start, and the player left in the
			// middle of a match, they'd lose that money. This way, they can quit in the middle of
			// the match, and come back to the same level of money that they started with
			this.mainPlayer.totalMoney	-= this.m_opponent.buyIn;
			this.mainPlayer.totalMoney 	+= this.mainPlayer.money;
			Console.print( "[StatePlay] Adding " + this.mainPlayer.money + " to the main player's total money; result: " + this.mainPlayer.totalMoney );
			
			// remove our cash callbacks to our players
			this.mainPlayer.onIsDealer	-= this._onPlayerIsDealer;
			this.mainPlayer.onPayCash 	-= this._onMainPlayerPayCash;
			this.mainPlayer.onWinCash 	-= this._onMainPlayerWinCash;
			this.mainPlayer.onWinHand 	-= this._onMainPlayerWinHand;
			this.m_opponent.onIsDealer	-= this._onPlayerIsDealer;
			this.m_opponent.onPayCash 	-= this._onOpponentPayCash;
			this.m_opponent.onWinCash 	-= this._onOpponentWinCash;
			this.m_opponent.onWinHand 	-= this._onOpponentWinHand;
			
			// remove our poker state manager delegate listener
			this.m_pokerStateManager.onStateChanged -= this._onPokerStateChange;
			
			// remove our buttons and msg bar
			this.m_buttons.hide();
			this.m_msgBar.hide();
			
			// stop any tweens
			Tween.killTweensOf( this.m_opponentGraphics );
			Tween.killTweensOf( this.m_table );
			Tween.killTweensOf( this.m_dealerButton );
			
			// tell the opponent graphics that we're leaving
			this.m_opponentGraphics.onLeaving();
			
			// set the anim finish callback on the pots
			this.m_pot.onFinishAnim += this._onPotEndTweensFinished;
			
			// tween out the player and the money stacks, then the rest
			var time:Number = 0.5;
			Tween.to( this.m_opponentGraphics, time, { "x":this.guiCanvas.designWidth, "ease":EaseType.EASE_IN } );
			Tween.to( this.m_dealerButton, time, { "alpha":0.0 } );
			this.m_potPlayer.currAmount 	= 0;
			this.m_potOpponent.currAmount	= 0;
			this.m_pot.currAmount			= 0;
		}
		
		/**********************************************************************/
		
		// called when all the begin() tweens have finished
		private function _onBeginTweensFinished( t:Tween ):void
		{
			// set our pot begin tween finished callback
			this.m_potPlayer.onFinishAnim += this._onPotBeginTweensFinished;
			
			// set our pots going and bring in our player
			Tween.to( this.m_opponentGraphics, 0.5, { "x":410.0, "ease":EaseType.EASE_OUT } ).onComplete += this._onBeginPlayerEnter;
			this.m_potPlayer.currAmount 	= this.mainPlayer.money;
			this.m_potOpponent.currAmount	= this.m_opponent.money;
		}
		
		// called when the player graphics have entered
		private function _onBeginPlayerEnter( t:Tween ):void
		{
			this.m_opponentGraphics.onEnter();
		}
		
		// called when the pot beginning tweens have finished
		private function _onPotBeginTweensFinished():void
		{
			// remove our callback
			this.m_potPlayer.onFinishAnim -= this._onPotBeginTweensFinished;
			
			// we're ready to start
			Console.print( "[StatePlay] Ready to play! Choosing the dealer!" );
			this.m_pokerStateManager.changeState( PokerStates.DEALER );
		}
		
		// called when the player/pot end tweens have finished
		private function _onPotEndTweensFinished():void
		{
			this.m_pot.onFinishAnim -= this._onPotEndTweensFinished;
			Tween.to( this.m_table, 0.5, { "y":0, "ease":EaseType.EASE_IN } ).onComplete += this._onEndTweensFinished;
		}
		
		// called when all the end() tweens have finished and we can change states
		private function _onEndTweensFinished( t:Tween ):void
		{
			// remove our elements
			this.guiCanvas.removeChild( this.m_bg, false );
			this.guiCanvas.removeChild( this.m_opponentGraphics, false );
			this.guiCanvas.removeChild( this.m_table, false );
			this.guiCanvas.removeChild( this.m_potPlayer, false );
			this.guiCanvas.removeChild( this.m_potOpponent, false );
			this.guiCanvas.removeChild( this.m_dealerButton, false );
			this.guiCanvas.removeChild( this.m_pot, false );
			this.guiCanvas.removeChild( this.m_msgBar, false );
				
			// finish
			this.onEnd();
		}
		
		// called when one of the players becomes the dealer
		private function _onPlayerIsDealer( player:Player ):void
		{
			this.m_dealerButton.opponentIsDealer = ( player == this.m_opponent );
		}
		
		// called when the main player pays some cash
		private function _onMainPlayerPayCash( amount:int ):void
		{
			// remove the cash from the player and set it in the main pot
			this.m_potPlayer.currAmount = this.mainPlayer.money;
			this.m_pot.currAmount		+= amount;
			
			// update our buttons max bet
			this.m_buttons.maxBet = this.mainPlayer.money;
		}
		
		// called when the main player wins some cash
		private function _onMainPlayerWinCash( amount:int ):void
		{
			// set our pot current amounts
			this.m_pot.currAmount		-= amount;
			this.m_potPlayer.currAmount	= this.mainPlayer.money;
			
			// update our buttons max bet
			this.m_buttons.maxBet = this.mainPlayer.money;
		}
		
		// called when the main player wins the thand
		private function _onMainPlayerWinHand( results:CardMatchResults, opponentFolded:Boolean, splitPot:Boolean ):void
		{			
			// show the msg bar
			if( splitPot )
				this.m_msgBar.show( "Split pot! You should have bluffed better!" );
			else if( opponentFolded )
				this.m_msgBar.show( "You win, as " + this.m_opponent.name + " folded, the wuss" );
			else
				this.m_msgBar.show( "You win, with " + results.getWinString() );
		}
		
		// called when the opponent pays some cash
		private function _onOpponentPayCash( amount:int ):void
		{
			// remove the cash from the opponent and set it in the main pot
			this.m_potOpponent.currAmount	= this.m_opponent.money;
			this.m_pot.currAmount			+= amount;
		}
		
		// called when the opponent wins some cash
		private function _onOpponentWinCash( amount:int ):void
		{
			// set our pot current amounts
			this.m_pot.currAmount			-= amount;
			this.m_potOpponent.currAmount	= this.m_opponent.money;
		}
		
		// called when the opponent wins the hand
		private function _onOpponentWinHand( results:CardMatchResults, playerFolded:Boolean, splitPot:Boolean ):void
		{			
			// show the msg bar
			if( splitPot )
				return; // if it's a split pot, then we show a message for the player
			else if( playerFolded )
				this.m_msgBar.show( this.m_opponent.name + " wins, as you folded, ya baby" );
			else
				this.m_msgBar.show( this.m_opponent.name + " wins, with " + results.getWinString() );
		}
		
		// called when the poker state manager has changed states
		private function _onPokerStateChange( oldState:GameState, newState:GameState ):void
		{
			// if we left the end match, then the match is over, switch back to our menu
			if( oldState != null && oldState is PokerStateEndMatch )
				this.stateManager.changeState( States.MENU );
		}
		
	}
	
}